# podflow/upload/login.py
# coding: utf-8

import os
import json
import uuid
import hashlib
from podflow import gVar
from podflow.upload.time_key import time_key
from podflow.basic.file_save import file_save
from podflow.basic.write_log import write_log
from podflow.basic.time_print import time_print
from podflow.basic.http_client import http_client


def get_login():
    try:
        with open("channel_data/upload_login.json", "r") as file:
            upload_data = file.read()
        gVar.upload_data = json.loads(upload_data)
    except Exception:
        file_save(gVar.upload_data, "upload_login.json", "channel_data")


def create():
    new_username = str(uuid.uuid4())
    while new_username in gVar.upload_data:
        new_username = str(uuid.uuid4())
    new_password = hashlib.sha256(os.urandom(64)).hexdigest()
    gVar.upload_data[new_username] = new_password
    file_save(gVar.upload_data, "upload_login.json", "channel_data")
    return new_username, new_password


def get_account(url):
    url = f"{url}/newuser"
    token = time_key(
        "We need to generate an account password for uploading non one-time items that need to be saved."
    )
    data = {"token": token}
    if response := http_client(
        url=url,
        name="获取上传服务账号密码",
        data=data,
    ):
        return response.json()


def login(url, username, password):
    url = f"{url}/login"
    data = {
        "username": username,
        "password": password,
    }
    if response := http_client(
        url=url,
        name="登陆上传服务",
        data=data,
    ):
        return response.json()


def login_upload(url):
    try:
        # 尝试打开并读取 JSON 文件
        with open("channel_data/upload_data.json", "r") as file:
            upload_json = file.read()
        upload_json = json.loads(
            upload_json
        )
    except Exception:
        upload_json = {}
    if "username" not in upload_json:
        write_log("上传服务账号密码不存在")
        time_print("获取上传服务账号密码...")
        account_data = get_account(url)
        if "code" in account_data:
            if account_data["code"] == 0:
                write_log("账号密码获取\033[32m成功\033[0m")
                username = account_data["data"]["username"]
                password = account_data["data"]["password"]
                upload_json = {
                    "username": username,
                    "password": password,
                }
                file_save(upload_json, "upload_data.json", "channel_data")
                return upload_json
            elif account_data["code"] == -1:
                write_log("账号密码获取\033[31m失败\033[0m: 认证失败")
                return
            else:
                write_log("账号密码获取\033[31m失败\033[0m")
                return
        else:
            write_log("账号密码获取\033[31m失败\033[0m: 无法连接")
            return
    else:
        username = upload_json["username"]
        password = upload_json.get("password", "")
        login_data = login(url, username, password)
        if "code" in login_data:
            if login_data["code"] == 0:
                time_print("登陆上传服务\033[32m成功\033[0m")
                return upload_json
            elif login_data["code"] == -1:
                write_log("登陆上传服务\033[31m失败\033[0m: 认证失败")
                return
            elif login_data["code"] == -2:
                write_log("登陆上传服务\033[31m失败\033[0m: 账号错误")
                return
            elif login_data["code"] == -3:
                write_log("登陆上传服务\033[31m失败\033[0m: 密码错误")
                return
            else:
                write_log("登陆上传服务\033[31m失败\033[0m")
                return
        else:
            write_log("登陆上传服务\033[31m失败\033[0m: 无法连接")
            return



# podflow/config/correct_channelid.py
# coding: utf-8

import re
from podflow import gVar, default_config, parse
from podflow.basic.write_log import write_log


# channelid修正模块
def correct_channelid(channelid, website):
    config = gVar.config
    channelid_name = ""
    output_name = ""
    if website == "youtube":
        channelid_name = "youtube"
        output_name = "YouTube"
    elif website == "bilibili":
        channelid_name = "哔哩哔哩弹幕网"
        output_name = "BiliBili"
    # 音视频格式及分辨率常量
    video_media = [
        "m4v",
        "mov",
        "qt",
        "avi",
        "flv",
        "wmv",
        "asf",
        "mpeg",
        "mpg",
        "vob",
        "mkv",
        "rm",
        "rmvb",
        "vob",
        "ts",
        "dat",
    ]
    dpi = [
        "144",
        "180",
        "216",
        "240",
        "360",
        "480",
        "720",
        "1080",
        "1440",
        "2160",
        "4320",
    ]
    media = ["m4a", "mp4"]

    # 判断正则表达式是否有效
    def is_valid_regex(pattern):
        try:
            re.compile(pattern)
            return True
        except re.error:
            return False

    # 复制字典channelid, 遍历复制后的字典进行操作以避免在循环中删除元素导致的迭代错误
    channelid_copy = channelid.copy()
    # 对channelid的错误进行更正
    for channelid_key, channeli_value in channelid_copy.items():
        # 判断是否为字典
        if not isinstance(channeli_value, dict):
            channeli_value = {"id": channeli_value}
            channelid[channelid_key] = channeli_value
        # 判断id是否正确
        if (
            "id" not in channeli_value
            or (
                website == "youtube"
                and not re.search(r"^UC.{22}", channeli_value["id"])
            )
            or (website == "bilibili" and not channeli_value["id"].isdigit())
        ):
            # 删除错误的
            del channelid[channelid_key]
            write_log(f"{output_name}频道 {channelid_key} ID不正确")
        else:
            # 对update_size进行纠正
            if (
                "update_size" not in channeli_value
                or not isinstance(channeli_value["update_size"], int)
                or channeli_value["update_size"] <= 0
            ):
                channelid[channelid_key]["update_size"] = default_config[
                    f"channelid_{website}"
                ][channelid_name]["update_size"]
            # 对id进行纠正
            if website == "youtube":
                channelid[channelid_key]["id"] = re.search(
                    r"UC.{22}", channeli_value["id"]
                ).group()
            # 对last_size进行纠正
            if (
                "last_size" not in channeli_value
                or not isinstance(channeli_value["last_size"], int)
                or channeli_value["last_size"] <= 0
            ):
                channelid[channelid_key]["last_size"] = default_config[
                    f"channelid_{website}"
                ][channelid_name]["last_size"]
            channelid[channelid_key]["last_size"] = max(
                channelid[channelid_key]["last_size"],
                channelid[channelid_key]["update_size"],
            )
            # 对title进行纠正
            if "title" not in channeli_value:
                channelid[channelid_key]["title"] = channelid_key
            # 对quality进行纠正
            if (
                (
                    "quality" not in channeli_value
                    or channeli_value["quality"] not in dpi
                )
                and "media" in channeli_value
                and channeli_value["media"] == "mp4"
            ):
                channelid[channelid_key]["quality"] = default_config[
                    f"channelid_{website}"
                ][channelid_name]["quality"]
            # 对media进行纠正
            if (
                "media" in channeli_value
                and channeli_value["media"] not in media
                and channeli_value["media"] in video_media
            ):
                channelid[channelid_key]["media"] = "mp4"
            elif (
                "media" in channeli_value
                and channeli_value["media"] not in media
                or "media" not in channeli_value
            ):
                channelid[channelid_key]["media"] = "m4a"
            # 对DisplayRSSaddress进行纠正
            if "DisplayRSSaddress" not in channeli_value or not isinstance(
                channeli_value["DisplayRSSaddress"], bool
            ):
                channelid[channelid_key]["DisplayRSSaddress"] = False
            # 对InmainRSS进行纠正
            if "InmainRSS" in channeli_value and isinstance(
                channeli_value["InmainRSS"], bool
            ):
                if channeli_value["InmainRSS"] is False:
                    channelid[channelid_key]["DisplayRSSaddress"] = True
            else:
                channelid[channelid_key]["InmainRSS"] = True
            # 对QRcode进行纠正
            if "QRcode" not in channeli_value or not isinstance(
                channeli_value["QRcode"], bool
            ):
                channelid[channelid_key]["QRcode"] = False
            # 对BackwardUpdate进行纠正
            if "BackwardUpdate" not in channeli_value or not isinstance(
                channeli_value["BackwardUpdate"], bool
            ):
                channelid[channelid_key]["BackwardUpdate"] = False
            # 对BackwardUpdate_size进行纠正
            if channelid[channelid_key]["BackwardUpdate"] and (
                "BackwardUpdate_size" not in channeli_value
                or not isinstance(channeli_value["BackwardUpdate_size"], int)
                or channeli_value["BackwardUpdate_size"] <= 0
            ):
                channelid[channelid_key]["BackwardUpdate_size"] = default_config[
                    f"channelid_{website}"
                ][channelid_name]["BackwardUpdate_size"]
            # 对want_retry_count进行纠正
            if (
                "want_retry_count" not in channeli_value
                or not isinstance(channeli_value["want_retry_count"], int)
                or channeli_value["want_retry_count"] <= 0
            ):
                channelid[channelid_key]["want_retry_count"] = default_config[
                    f"channelid_{website}"
                ][channelid_name]["want_retry_count"]
            # 对title_change进行纠正
            if "title_change" in channeli_value:
                title_changes = channeli_value["title_change"]
                uphold_title_changes = []
                if isinstance(title_changes, list):
                    for title_change in title_changes:
                        if website == "bilibli" and "url" in title_change:
                            del title_change["url"]
                        if (
                            isinstance(title_change, dict)
                            and "mode" in title_change
                            and "text" in title_change
                            and ("url" in title_change or "match" in title_change)
                        ):
                            mode = title_change["mode"]
                            match_url_pattern = r"https://www\.youtube\.com/playlist\?list=PL[0-9a-zA-Z_-]{32}"
                            if "url" in title_change and (
                                mode not in ["add-left", "add-right"]
                                or not re.match(match_url_pattern, title_change["url"])
                            ):
                                break
                            if "match" in title_change and (
                                mode not in ["add-left", "add-right", "replace"]
                                or not is_valid_regex(title_change["match"])
                            ):
                                break
                            uphold_title_changes.append(title_change)
                if uphold_title_changes:
                    channelid[channelid_key]["title_change"] = uphold_title_changes
                else:
                    del channelid[channelid_key]["title_change"]
            if website == "bilibili" and (
                "AllPartGet" not in channeli_value
                or not isinstance(channeli_value["AllPartGet"], bool)
            ):
                channelid[channelid_key]["AllPartGet"] = (
                    channelid[channelid_key]["update_size"] > 5
                )
            if website == "youtube" and (
                "NoShorts" not in channeli_value
                or not isinstance(channeli_value["NoShorts"], bool)
            ):
                channelid[channelid_key]["NoShorts"] = False
        if (
            channelid[channelid_key]["InmainRSS"] is False
            and f"{config['address']}/channel_rss/{channeli_value['id']}.xml"
            not in parse.shortcuts_url_original
        ):
            gVar.shortcuts_url[channelid_key] = (
                f"{config['address']}/channel_rss/{channeli_value['id']}.xml"
            )
    return channelid



# podflow/config/get_channelid.py
# coding: utf-8

from podflow import gVar
from podflow.basic.write_log import write_log
from podflow.basic.time_print import time_print


# 从配置文件中获取频道模块
def get_channelid(name):
    config = gVar.config
    output_name = ""
    if name == "youtube":
        output_name = "YouTube"
    elif name == "bilibili":
        output_name = "BiliBili"
    if f"channelid_{name}" in config:
        time_print(f"已读取{output_name}频道信息")
        return config[f"channelid_{name}"]
    else:
        write_log(f"{output_name}频道信息不存在")
        return {}



# podflow/download/show_progress.py
# coding: utf-8

from podflow.basic.time_print import time_print
from podflow.basic.time_format import time_format
from podflow.download.convert_bytes import convert_bytes


# 下载显示模块
def show_progress(stream):
    stream = dict(stream)
    if "downloaded_bytes" in stream:
        downloaded_bytes = convert_bytes(stream["downloaded_bytes"]).rjust(9)
    else:
        downloaded_bytes = " Unknow B"
    if "total_bytes" in stream:
        total_bytes = convert_bytes(stream["total_bytes"])
    else:
        total_bytes = "Unknow B"
    if stream["speed"] is None:
        speed = " Unknow B"
    else:
        speed = convert_bytes(stream["speed"], [" B", "KiB", "MiB", "GiB"], 1000).rjust(
            9
        )
    if stream["status"] in ["downloading", "error"]:
        if "total_bytes" in stream:
            percent = stream["downloaded_bytes"] / stream["total_bytes"] * 100
        else:
            percent = 0
        percent = f"{percent:.1f}" if percent == 100 else f"{percent:.2f}"
        percent = percent.rjust(5)
        eta = time_format(stream["eta"]).ljust(8)
        time_print(
            f"\033[94m{percent}%\033[0m|{downloaded_bytes}/{total_bytes}|\033[32m{speed}/s\033[0m|\033[93m{eta}\033[0m",
            Top=True,
            NoEnter=True,
            Time=False,
        )
    if stream["status"] == "finished":
        if "elapsed" in stream:
            elapsed = time_format(stream["elapsed"]).ljust(8)
        else:
            elapsed = "Unknown "
        time_print(
            f"100.0%|{downloaded_bytes}/{total_bytes}|\033[32m{speed}/s\033[0m|\033[97m{elapsed}\033[0m",
            Top=True,
            Time=False,
        )



# podflow/upload/upload_server.py
# coding: utf-8

import json


def get_uploads_original():
    try:
        # 尝试打开并读取 JSON 文件
        with open("channel_data/upload_server.json", "r") as file:
            uploads_original = file.read()  # 读取原始上传数据
        uploads_original = json.loads(
            uploads_original
        )  # 将读取的字符串转换为 Python 对象（列表或字典）
    except Exception:
        # 如果读取或解析失败，将 upload_original 初始化为空列表
        uploads_original = []



# podflow/message/want_retry.py
# coding: utf-8

import re


# 判断是否重试模块
def want_retry(video_url, num=1):
    # 定义正则表达式模式（不区分大小写）
    pattern = rf'\|{video_url}\|(试看|跳过更新|删除或受限|充电专属|直播预约\|a few moments\.)'
    # 读取 Podflow.log 文件
    try:
        with open('Podflow.log', 'r', encoding='utf-8') as file:
            content = file.read()  # 读取文件内容
        # 使用 re.findall() 查找所有匹配项
        matches = re.findall(pattern, content)
        # 计算匹配的个数
        count = len(matches)
    except Exception:
        count = 0
    return count < num or count % num == 0



# podflow/basic/http_client.py
# coding: utf-8

import time
import requests
from podflow.basic.time_print import time_print


# HTTP 请求重试模块
def http_client(
    url,
    name="",
    max_retries=10,
    retry_delay=4,
    headers_possess=False,
    cookies=None,
    data=None,
    mode="get",
    filename=None,
):
    user_agent = {
        "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/118.0.0.0 Safari/537.36"
    }
    if "bilibili" in url:
        user_agent["Referer"] = "https://www.bilibili.com/"
    elif "youtube" in url:
        user_agent["Referer"] = "https://www.youtube.com/"
    elif "douyin" in url:
        headers_douyin = {
            "authority": "sso.douyin.com",
            "accept": "application/json, text/plain, */*",
            "accept-language": "zh-CN,zh;q=0.9",
            "origin": "https://www.douyin.com",
            "referer": "https://www.douyin.com/",
            "sec-ch-ua": '"Google Chrome";v="117", "Not;A=Brand";v="8", "Chromium";v="117"',
            "sec-ch-ua-mobile": "?0",
            "sec-ch-ua-platform": '"Windows"',
            "sec-fetch-dest": "empty",
            "sec-fetch-mode": "cors",
            "sec-fetch-site": "same-site",
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/117.0.0.0 Safari/537.36",
        }
        user_agent |= headers_douyin
    err = None  # 初始化 err 变量
    response = None  # 初始化 response 变量
    # 创建一个Session对象
    session = requests.Session()
    if headers_possess:
        session.headers.update(user_agent)
    if cookies:
        session.cookies.update(cookies)
    if data:
        session.params.update(data)
    for num in range(max_retries):
        try:
            if mode.lower() != "post":
                response = session.get(url, timeout=8)
            elif filename:
                with open(filename, "rb") as f:
                    files = {"file": f}  # 这里 "file" 对应服务器端接收文件的字段名称
                    response = session.post(url, files=files, timeout=8)
            else:
                response = session.post(url, timeout=8)
            response.raise_for_status()
        except Exception as http_get_error:
            if response is not None and response.status_code in {404}:
                return response
            if name:
                time_print(f"{name}|\033[31m连接异常重试中...\033[97m{num + 1}\033[0m")
            if err:
                err = f"{err}\n{str(http_get_error)}"
            else:
                err = f":\n{str(http_get_error)}"
        else:
            return response
        time.sleep(retry_delay)
    if name:
        time_print(f"{name}|\033[31m达到最大重试次数\033[97m{max_retries}\033[0m{err}")
    return response



# podflow/main_upload.py
# coding: utf-8

import sys
import cherrypy
from podflow.upload.login import get_login
from podflow.basic.time_print import time_print
from podflow.basic.folder_build import folder_build
from podflow.httpfs.app_bottle import bottle_app_instance
from podflow.upload.linked_server import handle_discovery, usable_port


def main_upload():
    # 构建文件夹channel_audiovisual
    folder_build("channel_audiovisual")
    # 构建文件夹channel_data
    folder_build("channel_data")
    # 获取账号密码
    get_login()
    # 服务发现相关配置
    broadcast_port = 37001  # 服务发现用端口
    service_port = 5000  # 实际服务端口
    hostip = "0.0.0.0"

    broadcast_port = usable_port(broadcast_port, 37010)
    service_port = usable_port(service_port, 5010)

    if broadcast_port and service_port:
        # 设置路由
        bottle_app_instance.setup_routes(upload=True)
        # 设置logname
        bottle_app_instance.set_logname(
            logname="upload.log",
            http_fs=True,
        )
        # 启动 CherryPy 服务器
        cherrypy.tree.graft(
            bottle_app_instance.app_bottle
        )  # 将 Bottle 应用嵌入到 CherryPy 中
        cherrypy.config.update(
            {
                "global": {
                    "tools.sessions.on": True,  # 启用会话支持
                    "server.socket_host": hostip,  # 监听所有 IP 地址
                    "server.socket_port": service_port,  # 设置监听端口
                    "log.screen": False,  # 禁用屏幕日志输出
                    "log.access_file": "",  # 关闭访问日志
                    "log.error_file": "",  # 关闭错误日志
                }
            }
        )
        cherrypy.engine.start()  # 启动 CherryPy 服务器
        time_print(f"上传服务已启动|端口: \033[32m{service_port}\033[0m")
        # 服务发现
        handle_discovery(broadcast_port, service_port)
    else:
        if not broadcast_port:
            time_print("\033[31m广播端口被占用\033[97m(37001-37010)\033[0m")
        if not service_port:
            time_print("\033[31m服务端口被占用\033[97m(5000-5010)\033[0m")
        time_print("请清理被占用端口后重试")
        sys.exit(0)



# podflow/upload/linked_client.py
# coding: utf-8

import os
import time
import socket
from datetime import datetime
from podflow import gVar
from podflow.upload.time_key import time_key
from podflow.basic.time_print import time_print
from podflow.httpfs.progress_bar import progress_update


BROADCAST_PORT = 37001
TIMEOUT = 1  # 搜索超时时间（秒）
MAX_BROADCAST_PORT = 37010  # 尝试广播的最大端口


# 发现局域网内的服务器
def discover_server(broadcast_port, time_out):
    servers = []

    # 创建UDP socket
    with socket.socket(socket.AF_INET, socket.SOCK_DGRAM) as sock:
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
        sock.settimeout(time_out)
        send_text = time_key("PODFLOW_DISCOVER_SERVER_REQUEST")
        send_text = send_text.encode("utf-8")

        try:
            # 发送广播请求
            sock.sendto(send_text, ("<broadcast>", broadcast_port))
        except Exception:
            time_print("\033[31m请求发送失败\033[0m", False, True, False)
            return servers

        # 等待响应
        start_time = time.time()
        while time.time() - start_time < time_out:
            try:
                data, addr = sock.recvfrom(1024)
                if data.startswith(b"PODFLOW_SERVER_INFO|"):
                    try:
                        port = int(data.decode().split("|")[1])
                        servers.append((addr[0], port))
                    except (IndexError, ValueError):
                        time_print("\033[31m响应格式错误\033[0m", False, True, False)
            except socket.timeout:
                break
            except Exception:
                time_print("\033[31m接收数据出错\033[0m", False, True, False)
                break
    return servers


# 自动发现并连接服务器模块
def connect_upload_server():
    # 如果配置中启用了上传功能
    if gVar.config["upload"]:
        # 打印正在搜索上传服务器
        time_print("正在搜索上传服务器...")
        # 当前端口设置为广播端口
        current_port = BROADCAST_PORT
        # 服务器列表为空
        servers = []
        # 获取当前时间
        time_text = f"{datetime.now().strftime('%H:%M:%S')}|"
        # 获取命令行字节宽度
        try:
            terminal_width = os.get_terminal_size().columns
        except OSError:
            terminal_width = 47
        # 在允许的端口范围内尝试发现服务器
        while current_port < MAX_BROADCAST_PORT + 1:
            # 清空终端
            time_print(" " * terminal_width, True, True, False)
            # 打印尝试广播端口
            time_print(f"{time_text}尝试广播端口{current_port}...", True, True, False)
            progress_update(0.0005, added=True)
            servers = discover_server(current_port, TIMEOUT)
            if servers:
                break
            current_port += 1
        time_print("", Time=False)
        if not servers:
            time_print("找不到上传服务器", True)
        else:
            # 选择第一个找到的服务器
            server_ip, server_port = servers[0]
            time_print(f"正在连接到{server_ip}:{server_port}...", True)
            return f"http://{server_ip}:{server_port}"



# podflow/message/fail_message_initialize.py
# coding: utf-8

import re

error_reason = [
    [
        r"Premieres in ",
        "\033[31m预播\033[0m|",
        "text",
    ],
    [
        r"This live event will begin in ",
        "\033[31m直播预约\033[0m|",
        "text",
    ],
    [
        r"Video unavailable. This video contains content from SME, who has blocked it in your country on copyright grounds",
        "\033[31m版权保护\033[0m",
        "text",
    ],
    [
        r"Premiere will begin shortly",
        "\033[31m马上开始首映\033[0m",
        "text",
    ],
    [
        r"Private video. Sign in if you've been granted access to this video",
        "\033[31m私享视频\033[0m",
        "text",
    ],
    [
        r"This video is available to this channel's members on level: .*? Join this channel to get access to members-only content and other exclusive perks\.",
        "\033[31m会员专享\033[0m",
        "regexp",
    ],
    [
        r"Join this channel to get access to members-only content like this video, and other exclusive perks.",
        "\033[31m会员视频\033[0m",
        "text",
    ],
    [
        r"Video unavailable. This video has been removed by the uploader",
        "\033[31m视频被删除\033[0m",
        "text",
    ],
    [
        r"Video unavailable. This video is no longer available because the YouTube account associated with this video has been terminated.",
        "\033[31m关联频道被终止\033[0m",
        "text",
    ],
    [
        r"Video unavailable",
        "\033[31m视频不可用\033[0m",
        "text",
    ],
    [
        r"This video has been removed by the uploader",
        "\033[31m发布者删除\033[0m",
        "text",
    ],
    [
        r"This video has been removed for violating YouTube's policy on harassment and bullying",
        "\033[31m违规视频\033[0m",
        "text",
    ],
    [
        r"This video is private. If the owner of this video has granted you access, please sign in.",
        "\033[31m私人视频\033[0m",
        "text",
    ],
    [
        r"This video is unavailable",
        "\033[31m无法观看\033[0m",
        "text",
    ],
    [
        r"The following content is not available on this app.. Watch on the latest version of YouTube.",
        "\033[31m需App\033[0m",
        "text",
    ],
    [
        r"This video may be deleted or geo-restricted. You might want to try a VPN or a proxy server (with --proxy)",
        "\033[31m删除或受限\033[0m",
        "text",
    ],
    [
        r"Sign in to confirm your age. This video may be inappropriate for some users. Use --cookies-from-browser or --cookies for the authentication. See  https://github.com/yt-dlp/yt-dlp/wiki/FAQ#how-do-i-pass-cookies-to-yt-dlp  for how to manually pass cookies. Also see  https://github.com/yt-dlp/yt-dlp/wiki/Extractors#exporting-youtube-cookies  for tips on effectively exporting YouTube cookies",
        "\033[31m年龄限制\033[0m",
        "text",
    ],
    [
        r"Sign in to confirm your age. This video may be inappropriate for some users.",
        "\033[31m年龄限制\033[0m",
        "text",
    ],
    [
        r"Failed to extract play info; please report this issue on  https://github.com/yt-dlp/yt-dlp/issues?q= , filling out the appropriate issue template. Confirm you are on the latest version using  yt-dlp -U",
        "\033[31mInfo失败\033[0m",
        "text",
    ],
    [
        r"This is a supporter-only video: 该视频为「专属视频」专属视频，开通「[0-9]+元档包月充电」即可观看\. Use --cookies-from-browser or --cookies for the authentication\. See  https://github\.com/yt-dlp/yt-dlp/wiki/FAQ#how-do-i-pass-cookies-to-yt-dlp  for how to manually pass cookies",
        "\033[31m充电专属\033[0m",
        "regexp",
    ],
    [
        r"'.+' does not look like a Netscape format cookies file",
        "\033[31mCookie错误\033[0m",
        "regexp",
    ],
    [
        r"Sign in to confirm you’re not a bot. Use --cookies-from-browser or --cookies for the authentication. See  https://github.com/yt-dlp/yt-dlp/wiki/FAQ#how-do-i-pass-cookies-to-yt-dlp  for how to manually pass cookies. Also see  https://github.com/yt-dlp/yt-dlp/wiki/Extractors#exporting-youtube-cookies  for tips on effectively exporting YouTube cookies",
        "\033[31m需登录\033[0m",
        "text",
    ],
    [
        r"unable to download video data: HTTP Error 403: Forbidden",
        "\033[31m请求拒绝\033[0m",
        "text",
    ],
    [
        r"Got error: [0-9]* bytes read, [0-9]* more expected",
        "\033[31m数据不完整\033[0m",
        "regexp",
    ],
    [
        r"Got error: EOF occurred in violation of protocol (_ssl.c:992)",
        "\033[31m传输中断\033[0m",
        "text",
    ],
    [
        r"Got error: ('Connection aborted.', RemoteDisconnected('Remote end closed connection without response'))",
        "\033[31m请求超时\033[0m",
        "text",
    ],
    [
        r"Got error: HTTPSConnectionPool\(host='rr[0-9]---sn-.{8}\.googlevideo.com', port=443\): Read timed out\. \(read timeout=20\.0\)",
        "\033[31m响应超时\033[0m",
        "regexp",
    ],
    [
        r"Requested format is not available. Use --list-formats for a list of available formats",
        "\033[31m格式不可用\033[0m",
        "text",
    ],
    [
        r"Offline.",
        "\033[31m直播已停止\033[0m",
        "text",
    ]
]


# 失败信息初始化模块
def fail_message_initialize(message_error, video_url):
    if video_url[:2] == "BV":
        video_url = video_url[:12]
    fail_message = (
        str(message_error)
        .replace("ERROR: ", "")
        .replace("\033[0;31mERROR:\033[0m ", "")
        .replace(f"{video_url}: ", "")
        .replace("[youtube] ", "")
        .replace("[download] ", "")
        .replace("[BiliBili] ", "")
    )
    if video_url[:2] == "BV":
        fail_message = fail_message.replace(f"{video_url[2:]}: ", "")
    for fail_info, field, mode in error_reason:
        if mode == "text" and fail_info in fail_message:
            fail_message = fail_message.replace(f"{fail_info}", field)
            break
        elif mode == "regexp" and re.search(fail_info, fail_message):
            fail_message = re.sub(rf"{fail_info}", field, fail_message)
            break
    if fail_message[0] == "\n":
        fail_message = fail_message[1:]
    return fail_message



# podflow/basic/folder_build.py
# coding: utf-8

import os
from podflow.basic.write_log import write_log


# 构建文件夹模块
def folder_build(folder_name, parent_folder_name=None):
    if parent_folder_name:
        folder_path = os.path.join(os.getcwd(), parent_folder_name, folder_name)
    else:
        folder_path = os.path.join(os.getcwd(), folder_name)
    if not os.path.exists(folder_path):  # 判断文件夹是否存在
        os.makedirs(folder_path)  # 创建文件夹
        write_log(f"文件夹{folder_name}创建成功")



# podflow/bilibili/get.py
# coding: utf-8

import contextlib
import re
import json
import math
import time
import urllib
import threading
from hashlib import md5
from functools import reduce
from podflow import gVar
from podflow.basic.http_client import http_client
from podflow.httpfs.progress_bar import progress_bar
from podflow.basic.get_file_list import get_file_list
from podflow.basic.list_merge_tidy import list_merge_tidy


# WBI签名模块
def WBI_signature(params={}, img_key="", sub_key=""):
    mixinKeyEncTab = [
        46,
        47,
        18,
        2,
        53,
        8,
        23,
        32,
        15,
        50,
        10,
        31,
        58,
        3,
        45,
        35,
        27,
        43,
        5,
        49,
        33,
        9,
        42,
        19,
        29,
        28,
        14,
        39,
        12,
        38,
        41,
        13,
        37,
        48,
        7,
        16,
        24,
        55,
        40,
        61,
        26,
        17,
        0,
        1,
        60,
        51,
        30,
        4,
        22,
        25,
        54,
        21,
        56,
        59,
        6,
        63,
        57,
        62,
        11,
        36,
        20,
        34,
        44,
        52,
    ]

    def getMixinKey(orig: str):
        "对 imgKey 和 subKey 进行字符顺序打乱编码"
        return reduce(lambda s, i: s + orig[i], mixinKeyEncTab, "")[:32]

    def encWbi(params: dict, img_key: str, sub_key: str):
        "为请求参数进行 wbi 签名"
        mixin_key = getMixinKey(img_key + sub_key)
        curr_time = round(time.time())
        params["wts"] = curr_time  # 添加 wts 字段
        params = dict(sorted(params.items()))  # 按照 key 重排参数
        # 过滤 value 中的 "!'()*" 字符
        params = {
            k: "".join(filter(lambda chr: chr not in "!'()*", str(v)))
            for k, v in params.items()
        }
        query = urllib.parse.urlencode(params)  # 序列化参数
        wbi_sign = md5((query + mixin_key).encode()).hexdigest()  # 计算 w_rid
        params["w_rid"] = wbi_sign
        return params

    return encWbi(params=params, img_key=img_key, sub_key=sub_key)


# 获取bv的cid(包含分P或互动视频)模块
def get_bilibili_cid(bvid, bilibili_value):
    bvid_part = []  # 用于存储每个分P或互动视频的信息
    bvid_cid = []  # 存储已经获取的cid
    bvid_cid_choices = []  # 存储互动视频的选择
    code_num = {
        0: "成功",
        -400: "请求错误",
        -403: "权限不足",
        -404: "无视频",
        62002: "稿件不可见",
        62004: "稿件审核中",
        62012: "仅UP主自己可见",
    }

    # 获取互动视频信息模块
    def get_edge_info(bvid, bilibili_value, graph_version, edge_id):
        if edgeinfo_v2_response := http_client(
            "https://api.bilibili.com/x/stein/edgeinfo_v2",
            f"{bilibili_value}|{bvid}",
            10,
            4,
            True,
            None,
            {"bvid": bvid, "graph_version": graph_version, "edge_id": edge_id},
        ):
            edgeinfo_v2 = edgeinfo_v2_response.json()
            return edgeinfo_v2["data"]

    # 获取选择项信息模块
    def get_choices(data):
        options = []
        options_cid = []
        if "questions" in data["edges"]:
            for question in data["edges"]["questions"]:
                if "choices" in question:
                    for choice in question["choices"]:
                        if (
                            choice["cid"] not in bvid_cid
                            and choice["cid"] not in bvid_cid_choices
                        ):
                            bvid_cid_choices.append(
                                {
                                    "cid": choice["cid"],
                                    "edge_id": choice["id"],
                                    "option": choice["option"],
                                }
                            )
                            options.append(choice["option"])
                            options_cid.append(choice["cid"])
        return options, options_cid

    # 获取剧情图id模块
    def get_graph_version(bvid, bilibili_value, cid):
        graph_version = ""
        if playerwbi_response := http_client(
            "https://api.bilibili.com/x/player/wbi/v2",
            f"{bilibili_value}|{bvid}",
            10,
            4,
            True,
            None,
            {"cid": cid, "bvid": bvid},
        ):
            playerwbi = playerwbi_response.json()
            playerwbi_data = playerwbi["data"]
            if "interaction" in playerwbi_data:
                graph_version = playerwbi["data"]["interaction"]["graph_version"]
        return graph_version

    # 判断媒体类型
    if interface_response := http_client(
        "https://api.bilibili.com/x/web-interface/wbi/view",
        f"{bilibili_value}|{bvid}",
        10,
        4,
        True,
        gVar.bilibili_data["cookie"],
        {"bvid": bvid},
    ):
        interface_json = interface_response.json()
        code = interface_json["code"]
        if code != 0:
            error = code_num[code]
            return error, "error", None
        data = interface_json["data"]
        upower = data["is_upower_exclusive"]
        pages = data["pages"]
        cid = data["cid"]
        if len(pages) > 1:
            for part in pages:
                bvid_part.append(
                    {
                        "cid": part["cid"],
                        "page": part["page"],
                        "part": part["part"],
                        "duration": part["duration"],
                        "dimension": part["dimension"],
                        "first_frame": part["first_frame"],
                    }
                )
            bvid_part.sort(key=lambda x: x["page"], reverse=True)
            return bvid_part, "part", upower
        elif data["rights"]["is_stein_gate"]:
            # 获取互动视频信息
            if graph_version := get_graph_version(bvid, bilibili_value, cid):
                data_1 = get_edge_info(bvid, bilibili_value, graph_version, "1")
                for story_list in data_1["story_list"]:
                    if story_list["edge_id"] == 1:
                        story_list_1 = story_list
                        break
                options, options_cid = get_choices(data_1)
                bvid_part.append(
                    {
                        "cid": story_list_1["cid"],
                        "title": data_1["title"],
                        "edge_id": story_list_1["edge_id"],
                        "first_frame": f"http://i0.hdslb.com/bfs/steins-gate/{story_list_1['cid']}_screenshot.jpg",
                        "options": options,
                        "options_cid": options_cid,
                        "num": 1,
                    }
                )
                bvid_cid.append(story_list_1["cid"])
                while bvid_cid_choices:
                    if bvid_cid_choices[0]["cid"] not in bvid_cid:
                        data = get_edge_info(
                            bvid,
                            bilibili_value,
                            graph_version,
                            bvid_cid_choices[0]["edge_id"],
                        )
                        options, options_cid = get_choices(data)
                        bvid_part.append(
                            {
                                "cid": bvid_cid_choices[0]["cid"],
                                "title": data["title"],
                                "edge_id": bvid_cid_choices[0]["edge_id"],
                                "first_frame": f"http://i0.hdslb.com/bfs/steins-gate/{bvid_cid_choices[0]['cid']}_screenshot.jpg",
                                "options": options,
                                "options_cid": options_cid,
                                "num": len(bvid_part) + 1,
                            }
                        )
                        bvid_cid.append(bvid_cid_choices[0]["cid"])
                    del bvid_cid_choices[0]
                bvid_part.sort(key=lambda x: x["num"], reverse=True)
                return bvid_part, "edgeinfo", upower
            else:
                return None, None, None
        else:
            return cid, "cid", upower
    else:
        return None, None, None


# 查询哔哩哔哩用户投稿视频明细模块
def get_bilibili_vlist(bilibili_key, bilibili_value, num=1, all_part_judgement=False):
    bilibili_list = []
    bilibili_entry = {}
    if bilibili_response := http_client(
        "https://api.bilibili.com/x/space/wbi/arc/search",
        bilibili_value,
        10,
        4,
        True,
        gVar.bilibili_data["cookie"],
        WBI_signature(
            {
                "mid": bilibili_key,
                "pn": str(num),
                "ps": "25",
            },
            gVar.bilibili_data["img_key"],
            gVar.bilibili_data["sub_key"],
        ),
    ):
        bilibili_json = bilibili_response.json()
        bilibili_vlists = bilibili_json["data"]["list"]["vlist"]
        for vlist in bilibili_vlists:
            with contextlib.suppress(KeyError, TypeError, IndexError, ValueError):
                bilibili_entry[vlist["bvid"]] = {
                    "aid": vlist["aid"],
                    "author": vlist["author"],
                    "bvid": vlist["bvid"],
                    "copyright": vlist["copyright"],
                    "created": vlist["created"],
                    "description": vlist["description"],
                    "is_union_video": vlist["is_union_video"],
                    "length": vlist["length"],
                    "mid": vlist["mid"],
                    "pic": vlist["pic"],
                    "title": vlist["title"],
                    "typeid": vlist["typeid"],
                }
                bilibili_list.append(vlist["bvid"])
    if all_part_judgement and bilibili_list:

        def all_part(bvid):
            bvid_cid, bvid_type, power = get_bilibili_cid(bvid, bilibili_value)
            if bvid_type:
                bilibili_entry[bvid][bvid_type] = bvid_cid
                bilibili_entry[bvid]["power"] = power

        # 创建一个线程列表
        threads = []
        for bvid in bilibili_list:
            thread = threading.Thread(target=all_part, args=(bvid,))
            threads.append(thread)
            thread.start()
        # 等待所有线程完成
        for thread in threads:
            thread.join()
    return bilibili_entry, bilibili_list


# 更新哔哩哔哩频道json模块
def bilibili_json_update(bilibili_key, bilibili_value):
    bilibili_space = {}
    bilibili_lists = []
    bilibili_entrys = {}
    if not (
        bilibili_card_response := http_client(
            "https://api.bilibili.com/x/web-interface/card",
            bilibili_value,
            10,
            4,
            True,
            gVar.bilibili_data["cookie"],
            {
                "mid": bilibili_key,
                "photo": "true",
            },
        )
    ):
        return None
    bilibili_card_json = bilibili_card_response.json()
    with contextlib.suppress(KeyError, TypeError, IndexError, ValueError):
        if bilibili_card_json["code"] == 0:
            bilibili_space = {
                "mid": bilibili_card_json["data"]["card"]["mid"],
                "name": bilibili_card_json["data"]["card"]["name"],
                "sex": bilibili_card_json["data"]["card"]["sex"],
                "face": bilibili_card_json["data"]["card"]["face"],
                "spacesta": bilibili_card_json["data"]["card"]["spacesta"],
                "sign": bilibili_card_json["data"]["card"]["sign"],
                "Official": bilibili_card_json["data"]["card"]["Official"],
                "official_verify": bilibili_card_json["data"]["card"][
                    "official_verify"
                ],
            }
        else:
            return bilibili_card_json["code"]
    # 查询哔哩哔哩用户投稿视频明细
    for num in range(
        math.ceil(gVar.channelid_bilibili[bilibili_value]["update_size"] / 25)
    ):
        num += 1
        bilibili_entry, bilibili_list = get_bilibili_vlist(
            bilibili_key,
            f"{bilibili_value}第{num}页",
            num,
            gVar.channelid_bilibili[bilibili_value]["AllPartGet"],
        )
        bilibili_entrys = bilibili_entrys | bilibili_entry
        bilibili_lists += bilibili_list
    bilibili_space["entry"] = bilibili_entrys
    bilibili_space["list"] = bilibili_lists
    return bilibili_space


# 更新哔哩哔哩频道xml模块
def bilibili_rss_update(
    bilibili_key,
    bilibili_value,
    ratio_thread,
    rss_update_lock,
):
    bilibili_content_bvid_backward = []  # 初始化向后更新的内容列表
    last_size = gVar.channelid_bilibili[bilibili_value]["last_size"]
    # 获取已下载文件列表
    bilibili_content_bvid_original = get_file_list(
        bilibili_key, gVar.channelid_bilibili[bilibili_value]["media"]
    )[0]
    # 获取原xml中文件列表
    try:
        original_item = gVar.xmls_original[bilibili_key]  # 尝试获取原始的xml内容
        guids = list_merge_tidy(
            re.findall(r"(?<=<guid>).+(?=</guid>)", original_item), [], 12
        )  # 从xml中提取guid
    except KeyError:
        guids = []  # 如果没有找到对应的key，则初始化guids为空列表
    bilibili_space = bilibili_json_update(
        bilibili_key, bilibili_value
    )  # 更新bilibili相关的json内容
    # 读取原哔哩哔哩频道xml文件并判断是否要更新
    try:
        with open(
            f"channel_id/{bilibili_key}.json",
            "r",
            encoding="utf-8",  # 打开指定的json文件
        ) as file:
            bilibili_space_original = json.load(file)  # 读取文件内容并解析成字典
    except FileNotFoundError:  # 捕获文件不存在异常
        bilibili_space_original = {}  # 如果文件不存在，初始化为空字典
    except json.decoder.JSONDecodeError:  # 捕获json解码错误
        bilibili_space_original = {}  # 如果json读取失败，初始化为空字典
    # 根据更新条件更新频道数据
    if bilibili_space == -404:  # 检查更新状态
        gVar.channelid_bilibili_rss[bilibili_key] = {
            "content": bilibili_space,
            "type": "int",
        }  # 设置为整型内容
        bilibili_space_new = guids
    elif bilibili_space is None:
        gVar.channelid_bilibili_rss[bilibili_key] = {
            "content": bilibili_space_original,
            "type": "json",
        }  # 使用原始json内容
        bilibili_space_new = guids
    else:
        gVar.channelid_bilibili_rss[bilibili_key] = {
            "content": bilibili_space,
            "type": "dict",
        }  # 设置为字典类型内容
        # 判断是否需要更新ID列表
        if bilibili_space != bilibili_space_original:
            gVar.channelid_bilibili_ids_update[bilibili_key] = bilibili_value  # 更新ID
        # 获取需要更新的内容列表
        bilibili_content_bvid = bilibili_space["list"][
            : gVar.channelid_bilibili[bilibili_value]["update_size"]
        ]
        bilibili_space_new = list_merge_tidy(
            bilibili_content_bvid, guids
        )  # 合并新内容和原内容
        # 检查内容是否有变动
        if bilibili_content_bvid := [
            exclude
            for exclude in bilibili_content_bvid
            if exclude not in bilibili_content_bvid_original  # 筛选新增的内容
        ]:
            gVar.channelid_bilibili_ids_update[bilibili_key] = (
                bilibili_value  # 需要更新ID
            )
            gVar.bilibili_content_bvid_update[bilibili_key] = (
                bilibili_content_bvid  # 更新新增内容
            )
        # 向后更新
        if (
            gVar.channelid_bilibili[bilibili_value]["BackwardUpdate"] and guids
        ):  # 如果设置了向后更新
            backward_update_size = last_size - len(
                bilibili_space_new
            )  # 计算需要向后更新的数量
            if backward_update_size > 0:
                backward_update_size = min(
                    backward_update_size,
                    gVar.channelid_bilibili[bilibili_value]["BackwardUpdate_size"],
                )  # 限制更新数量
                backward_update_page_start = math.ceil(
                    len(bilibili_space_new) / 25
                )  # 确定开始页面
                backward_update_page_end = math.ceil(
                    (len(bilibili_space_new) + backward_update_size) / 25
                )  # 确定结束页面
                backward_entry = {}  # 初始化向后更新的条目
                backward_list = []  # 初始化向后更新的列表
                # 循环更新每一页的内容
                for num in range(
                    backward_update_page_start, backward_update_page_end + 1
                ):
                    backward_entry_part, backward_list_part = get_bilibili_vlist(
                        bilibili_key, bilibili_value, num
                    )  # 获取具体内容
                    backward_entry = backward_entry | backward_entry_part  # 合并条目
                    backward_list += backward_list_part  # 合并列表
                # 检查条目和列表是否有效
                if backward_entry and backward_list and guids[-1] in backward_list:
                    try:
                        backward_list_start = (
                            backward_list.index(guids[-1]) + 1
                        )  # 获取guids的起始索引
                        backward_list = backward_list[backward_list_start:][
                            :backward_update_size
                        ]  # 更新向后列表
                    except ValueError:
                        backward_list = []  # 如果没有找到，清空列表
                    # 根据条件移除已经存在的元素
                    for guid in backward_list.copy():
                        if guid in bilibili_space_new:
                            backward_list.remove(guid)  # 移除已存在的条目
                    # 如果有向后条目需要更新
                    if backward_list:
                        if gVar.channelid_bilibili[bilibili_value][
                            "AllPartGet"
                        ]:  # 如果需要获取所有部分

                            def backward_all_part(guid):
                                guid_cid, guid_type, power = get_bilibili_cid(
                                    guid, bilibili_value
                                )
                                if guid_type:
                                    backward_entry[guid][guid_type] = guid_cid
                                    backward_entry[guid]["power"] = power

                            # 创建一个线程列表
                            threads = []
                            for guid in backward_list:
                                thread = threading.Thread(
                                    target=backward_all_part, args=(guid,)
                                )  # 为每个条目创建线程
                                threads.append(thread)  # 添加线程到列表
                                thread.start()  # 启动线程
                            # 等待所有线程完成
                            for thread in threads:
                                thread.join()
                        # 更新频道信息
                        gVar.channelid_bilibili_rss[bilibili_key].update(
                            {
                                "backward": {
                                    "list": backward_list,
                                    "entry": backward_entry,
                                }
                            }
                        )
                        gVar.channelid_bilibili_ids_update[bilibili_key] = (
                            bilibili_value  # 标记ID更新
                        )
                        for guid in backward_list:
                            if (
                                guid not in bilibili_content_bvid_original
                            ):  # 检查新增的内容
                                bilibili_content_bvid_backward.append(
                                    guid
                                )  # 添加到向后更新列表
                        if bilibili_content_bvid_backward:
                            gVar.bilibili_content_bvid_backward_update[bilibili_key] = (
                                bilibili_content_bvid_backward  # 更新最终的向后更新内容
                            )
    gVar.xmls_quantity[bilibili_key] = min(last_size, len(bilibili_space_new)) + len(
        bilibili_content_bvid_backward
    )
    # 更新进度条
    with rss_update_lock:
        progress_bar(ratio_thread, 0.09)



# podflow/message/get_video_format_multithread.py
# coding: utf-8

import threading
from podflow.download.wait_animation import wait_animation
from podflow.message.get_youtube_and_bilibili_video_format import (
    get_youtube_and_bilibili_video_format,
)


# YouTube&哔哩哔哩获取视频信息多线程模块
def get_video_format_multithread(
    video_id_update_format_item,
    wait_animation_display_info,
    ratio_part,
):
    # 创建共享的标志变量
    stop_flag = ["keep"]  # 使用列表来存储标志变量
    # 创建两个线程分别运行等待动画和其他代码, 并传递共享的标志变量
    prepare_animation = threading.Thread(
        target=wait_animation,
        args=(
            stop_flag,
            wait_animation_display_info,
        ),
    )
    # 启动动画线程
    prepare_animation.start()
    # 创建线程锁
    video_format_lock = threading.Lock()
    ratio_part_lock = threading.Lock()
    # 创建线程列表
    video_id_update_threads = []
    
    for video_id in video_id_update_format_item.keys():
        thread = threading.Thread(
            target=get_youtube_and_bilibili_video_format,
            args=(
                video_id,
                stop_flag,
                video_format_lock,
                prepare_animation,
                ratio_part,
                ratio_part_lock,
                ),
        )
        video_id_update_threads.append(thread)
        thread.start()
    # 等待所有线程完成
    for thread in video_id_update_threads:
        thread.join()
    stop_flag[0] = "end"
    prepare_animation.join()



# podflow/remove/remove_file.py
# coding: utf-8

import os
from podflow import gVar
from podflow.basic.write_log import write_log


# 删除多余媒体文件模块
def remove_file():
    channelid_youtube_ids = gVar.channelid_youtube_ids

    for output_dir, name in channelid_youtube_ids.items():
        for file_name in os.listdir(f"channel_audiovisual/{output_dir}"):
            if file_name not in gVar.all_youtube_content_ytid[output_dir]:
                os.remove(f"channel_audiovisual/{output_dir}/{file_name}")
                write_log(f"{name}|{file_name}已删除")

    channelid_bilibili_ids = gVar.channelid_bilibili_ids
    for output_dir, name in channelid_bilibili_ids.items():
        for file_name in os.listdir(f"channel_audiovisual/{output_dir}"):
            if file_name not in gVar.all_bilibili_content_bvid[output_dir]:
                os.remove(f"channel_audiovisual/{output_dir}/{file_name}")
                write_log(f"{name}|{file_name}已删除")



# podflow/Netscape/get_cookie_dict.py
# coding: utf-8

from http.cookiejar import LoadError, MozillaCookieJar
from podflow.basic.time_print import time_print


# 将Netscape转Dict模块
def get_cookie_dict(file):
    parts = file.split("/")
    try:
        # 加载Netscape格式的cookie文件
        cookie_jar = MozillaCookieJar(file)
        cookie_jar.load(ignore_discard=True)
        return {cookie.name: cookie.value for cookie in cookie_jar}
    except FileNotFoundError:
        time_print(f"{parts[-1]}文件不存在")
        return None
    except LoadError:
        time_print(f"{parts[-1]}文件错误")
        return None



# podflow/download/wait_animation.py
# coding: utf-8

import time
from datetime import datetime
from podflow.basic.time_print import time_print


# 等待动画模块
def wait_animation(stop_flag, wait_animation_display_info):
    animation = "."
    i = 1
    prepare_youtube_print = datetime.now().strftime("%H:%M:%S")
    while True:
        if stop_flag[0] == "keep":
            time_print(
                f"{prepare_youtube_print}|{wait_animation_display_info}\033[34m准备中{animation.ljust(5)}\033[0m",
                Top=True,
                NoEnter=True,
                Time=False,
            )
            if i % 5 == 0:
                animation = "."
            else:
                animation += "."
            i += 1
            time.sleep(0.5)
        elif stop_flag[0] == "error":
            time_print(
                f"{prepare_youtube_print}|{wait_animation_display_info}\033[34m准备中{animation}\033[0m \033[31m失败:\033[0m",
                Top=True,
                Time=False,
            )
            break
        elif stop_flag[0] == "end":
            time_print(
                f"{prepare_youtube_print}|{wait_animation_display_info}\033[34m准备中{animation} 已完成\033[0m",
                Top=True,
                Time=False,
            )
            break



# podflow/download/youtube_and_bilibili_download.py
# coding: utf-8

from podflow import gVar
from podflow.basic.write_log import write_log
from podflow.download.dl_aideo_video import dl_aideo_video


# 下载YouTube和哔哩哔哩视频
def youtube_and_bilibili_download():
    for video_id, format_value in gVar.video_id_update_format.items():
        if isinstance(format_value, dict) and format_value["main"] not in gVar.video_id_failed:
            output_dir_name = format_value["name"]
            display_color = "\033[35m" if format_value["backward_update"] else "\033[95m"
            if dl_aideo_video(
                video_id,
                format_value["id"],
                format_value["media"],
                format_value["format"],
                gVar.config["retry_count"],
                format_value["download"]["url"],
                output_dir_name,
                format_value["cookie"],
                format_value["download"]["num"],
                display_color
            ):
                gVar.video_id_failed.append(format_value["main"])
                write_log(
                    f"{display_color}{output_dir_name}\033[0m|{video_id} \033[31m无法下载\033[0m"
                )
            gVar.video_id_update_format[video_id]["finish"] = True


# podflow/message/optimize_download.py
# coding: utf-8

from podflow import gVar
from podflow.basic.write_log import write_log
from podflow.basic.time_print import time_print


# 优化下载顺序模块
def optimize_download():
    xmls_quantity = gVar.xmls_quantity
    video_id_update_format = gVar.video_id_update_format
    sorted_video_id_update_format = {}
    time_print("开始优化下载顺序")
    # 按总和从大到小排序
    sorted_channels = sorted(xmls_quantity.items(), key=lambda x: x[1], reverse=True)
    # 根据总和排序数据
    for channel_id, _ in sorted_channels:
        for key, value in video_id_update_format.items():
            if value["id"] == channel_id:
                sorted_video_id_update_format[key] = value
    if len(video_id_update_format) == len(sorted_video_id_update_format):
        gVar.video_id_update_format = sorted_video_id_update_format
        time_print("下载顺序优化\033[32m成功\033[0m")
    else:
        write_log("下载顺序优化\033[31m失败\033[0m")



# podflow/youtube/build.py
# coding: utf-8

import re
import html
import threading
import contextlib
import xml.etree.ElementTree as ET
from datetime import datetime, timezone
from podflow import gVar
from podflow.message.xml_item import xml_item
from podflow.basic.time_print import time_print
from podflow.basic.http_client import http_client
from podflow.message.format_time import format_time
from podflow.httpfs.progress_bar import progress_bar
from podflow.basic.get_html_dict import get_html_dict
from podflow.message.xml_original_item import xml_original_item


# 获取YouTube频道简介模块
def get_youtube_introduction():
    # 创建线程锁
    youtube_xml_get_lock = threading.Lock()

    # 使用http获取youtube频道简介和图标模块
    def youtube_xml_get(output_dir):
        if channel_about := http_client(
            url=f"https://www.youtube.com/channel/{output_dir}/about",
            max_retries=2,
            retry_delay=5,
        ):
            channel_about = channel_about.text
            xml_tree = {
                "icon": re.sub(
                    r"=s(0|[1-9]\d{0,3}|1[0-9]{1,3}|20[0-3][0-9]|204[0-8])-c-k",
                    "=s2048-c-k",
                    re.search(
                        r"https?://yt3.googleusercontent.com/[^\s]*(?=\">)",
                        channel_about,
                    ).group(),
                )
            }
            xml_tree["description"] = re.search(
                r"(?<=\<meta itemprop\=\"description\" content\=\").*?(?=\")",
                channel_about,
                flags=re.DOTALL,
            ).group()
        else:
            xml_tree = {"introduction": False}
            with youtube_xml_get_lock:
                gVar.youtube_xml_get_tree[output_dir] = xml_tree
    # 创建线程列表
    youtube_xml_get_threads = []
    for output_dir in gVar.channelid_youtube_ids_update:
        thread = threading.Thread(target=youtube_xml_get, args=(output_dir,))
        youtube_xml_get_threads.append(thread)
        thread.start()
    # 等待所有线程完成
    for thread in youtube_xml_get_threads:
        thread.join()


# 打印无法获取youtube信息模块
def print_fail_youtube():
    for output_dir, xml_tree in gVar.youtube_xml_get_tree.items():
        if "introduction" in xml_tree:
            time_print(f"{gVar.channelid_youtube_ids[output_dir]} 简介获取失败")
        if "playlist" in xml_tree:
            time_print(f"{gVar.channelid_youtube_ids[output_dir]} 播放列表获取失败") 


# 获取YouTube播放列表模块
def get_youtube_playlist(url):
    videoids = []
    ytInitialData = get_html_dict(url, "", "ytInitialData")
    with contextlib.suppress(KeyError):
        contents = ytInitialData["contents"]["twoColumnBrowseResultsRenderer"]["tabs"][
            0
        ]["tabRenderer"]["content"]["sectionListRenderer"]["contents"][0][
            "itemSectionRenderer"
        ][
            "contents"
        ][
            0
        ][
            "playlistVideoListRenderer"
        ][
            "contents"
        ]
        videoids.extend(
            content["playlistVideoRenderer"]["videoId"] for content in contents
        )
    return videoids


# 从HTML获取YouTube媒体信息模块
def youtube_html_guid_message(guid):
    ytInitialPlayerResponse = get_html_dict(
        f"https://www.youtube.com/watch?v={guid}",
        f"{guid} HTML",
        "ytInitialPlayerResponse",
    )
    try:
        image = ytInitialPlayerResponse["microformat"]["playerMicroformatRenderer"][
            "thumbnail"
        ]["thumbnails"][0]["url"]
        title = ytInitialPlayerResponse["microformat"]["playerMicroformatRenderer"][
            "title"
        ]["simpleText"]
        description = ytInitialPlayerResponse["microformat"][
            "playerMicroformatRenderer"
        ]["description"]["simpleText"]
        published = ytInitialPlayerResponse["microformat"]["playerMicroformatRenderer"][
            "publishDate"
        ]
        return published, image, title, description
    except KeyError:
        return None


# 生成YouTube的item模块
def youtube_xml_item(entry, title_change=None):
    if title_change is None:
        title_change = {}

    # 输入时间字符串和原始时区
    time_str = re.search(r"(?<=<published>).+(?=</published>)", entry).group()
    pubDate = format_time(time_str)
    output_dir = re.search(r"(?<=<yt:channelId>).+(?=</yt:channelId>)", entry).group()
    description = re.search(
        r"(?<=<media:description>).+(?=</media:description>)",
        re.sub(r"\n+", "\n", entry),
        flags=re.DOTALL,
    )
    description = description.group() if description else ""
    ytid = re.search(r"(?<=<yt:videoId>).+(?=</yt:videoId>)", entry).group()
    return xml_item(
        ytid,
        output_dir,
        f"https://youtube.com/watch?v={ytid}",
        gVar.channelid_youtube[gVar.channelid_youtube_ids[output_dir]]["title"],
        re.search(r"(?<=<title>).+(?=</title>)", entry).group(),
        description,
        pubDate,
        re.search(r"(?<=<media:thumbnail url=\").+(?=\" width=\")", entry).group(),
        title_change,
    )


def get_xml_item(guid, item, channelid_title, title_change, output_dir):
    video_website = f"https://youtube.com/watch?v={guid}"
    if item["yt-dlp"]:
        guid_value = gVar.video_id_update_format[guid]
        if timestamp := guid_value["timestamp"]:
            published = datetime.fromtimestamp(timestamp, timezone.utc).strftime(
                "%Y-%m-%dT%H:%M:%S%z"
            )
            title = guid_value["title"]
            description = guid_value["description"]
            image = guid_value["image"]
        elif guid_message := youtube_html_guid_message(guid):
            published, image, title, description = guid_message
        else:
            return None
        title = html.escape(title)
        description = html.escape(re.sub(r"\n+", "\n", description))
        image = re.sub(r"\?.*$", "", image)
        pubDate = format_time(published)
    else:
        title = html.escape(item["title"])
        description = html.escape(re.sub(r"\n+", "\n", item["description"]))
        pubDate = format_time(item["pubDate"])
        image = re.sub(r"\?.*$", "", item["image"])
    return xml_item(
        guid,
        output_dir,
        video_website,
        channelid_title,
        title,
        description,
        pubDate,
        image,
        title_change,
    )


# 生成YouTube对应channel的需更新的items模块
def youtube_xml_items(output_dir, ratio_part):
    items_list = [f"<!-- {output_dir} -->"]
    entry_num = 0
    original_judgment = True
    channelid_youtube_value = gVar.channelid_youtube[
        gVar.channelid_youtube_ids[output_dir]
    ]
    channelid_title = channelid_youtube_value["title"]
    title_change = channelid_youtube_value.get("title_change", [])
    update_size = channelid_youtube_value["update_size"]
    if title_change:
        for title_index, title_value in enumerate(title_change):
            if "url" in title_value:
                title_index_table = get_youtube_playlist(title_value["url"])
                title_change[title_index]["table"] = title_index_table
                if not title_index_table:
                    if output_dir in gVar.youtube_xml_get_tree:
                        gVar.youtube_xml_get_tree[output_dir]["playlist"] = False
                    else:
                        gVar.youtube_xml_get_tree[output_dir] = {"playlist": False}
    output_dir_value = gVar.channelid_youtube_rss[output_dir]
    # 最新更新
    if output_dir_value["type"] == "dict":
        for guid in output_dir_value["content"]["list"]:
            if guid not in gVar.video_id_failed:
                item = output_dir_value["content"]["item"][guid]
                if xml_item_text := get_xml_item(
                    guid, item, channelid_title, title_change, output_dir
                ):
                    items_list.append(f"{xml_item_text}<!-- {output_dir} -->")
                    progress_bar(ratio_part[0], ratio_part[1])
                    if (
                        gVar.video_id_update_format[guid]["description"]
                        and gVar.video_id_update_format[guid]["description"][0] == "『"
                    ):
                        original_judgment = False
    else:
        if output_dir_value["type"] == "html":  # 获取最新的rss信息
            file_xml = output_dir_value["content"].text
        else:
            file_xml = output_dir_value["content"]
        entrys = re.findall(r"<entry>.+?</entry>", file_xml, re.DOTALL)
        for entry in entrys:
            if (
                re.search(r"(?<=<yt:videoId>).+(?=</yt:videoId>)", entry).group()
                not in gVar.video_id_failed
            ):
                items_list.append(
                    f"{youtube_xml_item(entry, title_change)}<!-- {output_dir} -->"
                )
                progress_bar(ratio_part[0], ratio_part[1])
                if re.search(r"(?<=<media:description>)『", entry):
                    original_judgment = False
            entry_num += 1
            if entry_num >= update_size:
                break
    items_guid = re.findall(r"(?<=<guid>).+?(?=</guid>)", "".join(items_list))
    # 存量接入
    entry_count = channelid_youtube_value["last_size"] - len(items_guid)
    if gVar.xmls_original and output_dir in gVar.xmls_original and entry_count > 0:
        xml_num = 0
        for xml in gVar.xmls_original[output_dir].split(f"<!-- {output_dir} -->"):
            xml_guid = re.search(r"(?<=<guid>).+(?=</guid>)", xml)
            if (
                xml_guid
                and xml_guid.group() not in items_guid
                and xml_guid.group() not in gVar.video_id_failed
            ):
                items_list.append(
                    f"{xml_original_item(xml, channelid_title, original_judgment, title_change)}<!-- {output_dir} -->"
                )
                progress_bar(ratio_part[0], ratio_part[1])
                xml_num += 1
            if xml_num >= entry_count:
                break
    # 向后更新
    with contextlib.suppress(KeyError):
        backward = output_dir_value["backward"]
        for backward_guid in backward["list"]:
            if backward_guid not in gVar.video_id_failed:
                backward_item = backward["item"][backward_guid]
                if backward_xml_item_text := get_xml_item(
                    backward_guid,
                    backward_item,
                    channelid_title,
                    title_change,
                    output_dir,
                ):
                    items_list.append(f"{backward_xml_item_text}<!-- {output_dir} -->")
                    progress_bar(ratio_part[0], ratio_part[1])
    # 生成对应xml
    try:
        with open(
            f"channel_rss/{output_dir}.xml", "r", encoding="utf-8"
        ) as file:  # 打开文件进行读取
            root = ET.parse(file).getroot()
            description = (root.findall(".//description")[0]).text
            description = "" if description is None else html.escape(description)
            icon = (root.findall(".//url")[0]).text
    except Exception:  # 参数不存在直接更新
        description = gVar.config["description"]
        icon = gVar.config["icon"]
    youtube_xml_get_tree = gVar.youtube_xml_get_tree
    if (
        output_dir in gVar.channelid_youtube_ids_update
        and output_dir in youtube_xml_get_tree
        and "introduction" not in youtube_xml_get_tree[output_dir]
    ):
        description = youtube_xml_get_tree[output_dir]["description"]
        icon = youtube_xml_get_tree[output_dir]["icon"]
    category = gVar.config["category"]
    if output_dir_value["type"] == "dict":
        title = output_dir_value["content"]["title"]
    else:
        title = re.search(r"(?<=<title>).+(?=</title>)", file_xml).group()
    link = f"https://www.youtube.com/channel/{output_dir}"
    items = "".join(items_list)
    items = f"""<!-- {{{output_dir}}} -->
{items}
<!-- {{{output_dir}}} -->"""
    return {
        "title": title,
        "link": link,
        "description": description,
        "category": category,
        "icon": icon,
        "items": items,
    }



# podflow/upload/get_upload_original.py
# coding: utf-8

import re
import json
from collections import Counter
from email.utils import parsedate_tz, mktime_tz
from podflow import gVar
from podflow.basic.file_save import file_save
from podflow.basic.write_log import write_log
from podflow.basic.time_print import time_print
from podflow.message.get_media_name import get_media_name


# 获取原始上传数据模块
def get_upload():
    xmls_original = gVar.xmls_original
    try:
        # 尝试打开并读取 JSON 文件
        with open("channel_data/upload.json", "r") as file:
            upload_original = file.read()  # 读取原始上传数据
        upload_original = json.loads(
            upload_original
        )  # 将读取的字符串转换为 Python 对象（列表或字典）
    except Exception:
        # 如果读取或解析失败，将 upload_original 初始化为空列表
        upload_original = []

    # 如果 upload_original 不为空
    if upload_original:
        # 提取每个条目的 channel_id
        channel_ids = [item.get("channel_id") for item in upload_original]
        # 统计每个 channel_id 的出现次数
        channelid_counts = Counter(channel_ids)
        # 将出现次数转换为字典
        age_counts = dict(channelid_counts)
        # 统计 xmls_original 中每个键对应的 <guid> 标签内的元素数量
        xmls_original_counts = {
            key: len(re.findall(r"(?<=<guid>).+(?=</guid>)", value))
            for key, value in xmls_original.items()
        }
        # 如果两个计数字典不相等，提示错误
        if age_counts != xmls_original_counts:
            time_print("无法获取完整原始上传信息")
    # 如果 upload_original 为空
    else:
        # 遍历 xmls_original 的每个键值对
        for xmls_original_key, xmls_original_value in xmls_original.items():
            # 如果当前键在 channelid_youtube_ids 中
            if xmls_original_key in gVar.channelid_youtube_ids:
                # 使用正则表达式解析包含特定格式媒体ID的字符串
                media = get_media_name("youtube", xmls_original_value)
            # 如果当前键在 channelid_bilibili_ids 中
            elif xmls_original_key in gVar.channelid_bilibili_ids:
                media = get_media_name("bilibili", xmls_original_value)
            else:
                media = []
            # 查找每个上传条目的发布时间
            upload_time = re.findall(
                r"(?<=<pubDate>).+(?=</pubDate>)",
                xmls_original_value,
            )
            # 将日期字符串列表转换为 Unix 时间戳列表
            timestamps = [
                mktime_tz(
                    parsedate_tz(date_string)
                )  # 解析和转换每个日期字符串为 Unix 时间戳
                for date_string in upload_time
                if parsedate_tz(date_string)  # 确保解析成功
            ]
            # 如果媒体和时间戳的数量不匹配，记录错误并清空 upload_original
            if len(media) != len(timestamps):
                write_log("获取原始上传内容失败")  # 错误日志
                upload_original.clear()  # 清空 upload_original
                break  # 退出循环
            # 如果数量匹配，则整合 media_id、channel_id 和上传时间到 upload_original 列表
            upload_original += [
                {
                    "media_id": key,
                    "channel_id": xmls_original_key,
                    "media_time": value,
                    "upload": False,
                    "remove": False,
                    "hash": None,
                }
                for value, key in zip(
                    timestamps, media
                )  # 使用 zip() 将 media 和 timestamps 组合成对
            ]
        # 如果成功填充 upload_original
        if upload_original:
            file_save(upload_original, "upload.json", "channel_data")  # 保存到文件
    return upload_original


# 初始化原始上传信息
def get_upload_original():
    if gVar.config["upload"]:
        if upload_original := get_upload():
            gVar.upload_original = upload_original
        else:
            gVar.config["upload"] = False



# podflow/basic/time_stamp.py
# coding: utf-8

import time
import threading
import contextlib
from podflow.basic.time_print import time_print
from podflow.basic.http_client import http_client


# 时间戳模块
def time_stamp():
    time_stamps = []

    # 获取时间戳淘宝
    def time_stamp_taobao():
        if response := http_client(
            "http://api.m.taobao.com/rest/api3.do?api=mtop.common.getTimestamp",
            "",
            1,
            0,
        ):
            response_json = response.json()
            with contextlib.suppress(KeyError):
                time_stamps.append(int(response_json["data"]["t"]))

    # 获取时间戳美团
    def time_stamp_meituan():
        if response := http_client(
            "https://cube.meituan.com/ipromotion/cube/toc/component/base/getServerCurrentTime",
            "",
            1,
            0,
        ):
            response_json = response.json()
            with contextlib.suppress(KeyError):
                time_stamps.append(int(response_json["data"]))

    # 获取时间戳苏宁
    def time_stamp_suning():
        if response := http_client("https://f.m.suning.com/api/ct.do", "", 1, 0):
            response_json = response.json()
            with contextlib.suppress(KeyError):
                time_stamps.append(int(response_json["currentTime"]))

    # 创建线程
    thread1 = threading.Thread(target=time_stamp_taobao)
    thread2 = threading.Thread(target=time_stamp_meituan)
    thread3 = threading.Thread(target=time_stamp_suning)
    # 启动线程
    thread1.start()
    thread2.start()
    thread3.start()
    # 等待线程结束
    thread1.join()
    thread2.join()
    thread3.join()
    if time_stamps:
        return int(sum(time_stamps) / len(time_stamps))
    time_print("\033[31m获取时间戳api失败\033[0m")
    return round(time.time() * 1000)



# podflow/message/update_youtube_bilibili_rss.py
# coding: utf-8

import re
import threading
from podflow import gVar
from podflow.basic.file_save import file_save
from podflow.basic.write_log import write_log
from podflow.youtube.get import youtube_rss_update
from podflow.basic.folder_build import folder_build
from podflow.bilibili.get import bilibili_rss_update
from podflow.httpfs.progress_bar import progress_bar


# 更新Youtube和哔哩哔哩频道xml多线程模块
def update_youtube_bilibili_rss():
    if gVar.channelid_youtube_ids or gVar.channelid_bilibili_ids:
        channelid_quantity = len(gVar.channelid_youtube_ids) + len(gVar.channelid_bilibili_ids)
        ratio_part = 0.01 / channelid_quantity
        ratio_thread = 0.05 / channelid_quantity
    else:
        ratio_part = 0
        ratio_thread = 0

    pattern_youtube404 = r"Error 404 \(Not Found\)"  # 设置要匹配的正则表达式模式
    pattern_youtube_error = {
        "This channel was removed because it violated our Community Guidelines.": "违反社区准则",
        "This channel does not exist.": "不存在 (ID错误) ",
    }
    pattern_youtube_varys = [
        r"[0-9]{4}-[0-9]{2}-[0-9]{2}T[0-2][0-9]:[0-6][0-9]:[0-6][0-9]\+00:00",
        r'starRating count="[0-9]*"',
        r'statistics views="[0-9]*"',
        r"<id>yt:channel:(UC)?(.{22})?</id>",
        r"<yt:channelId>(UC)?(.{22})?</yt:channelId>",
    ]
    youtube_bilibili_rss_update_threads = []  # 创建线程列表
    rss_update_lock = threading.Lock()
    # Youtube多线程
    for youtube_key, youtube_value in gVar.channelid_youtube_ids.items():
        thread = threading.Thread(
            target=youtube_rss_update,
            args=(
                youtube_key,
                youtube_value,
                pattern_youtube_varys,
                pattern_youtube404,
                pattern_youtube_error,
                ratio_thread,
                rss_update_lock,
            ),
        )
        youtube_bilibili_rss_update_threads.append(thread)
        # 开始多线程
        thread.start()
    # 哔哩哔哩多线程
    for bilibili_key, bilibili_value in gVar.channelid_bilibili_ids.items():
        thread = threading.Thread(
            target=bilibili_rss_update,
            args=(
                bilibili_key,
                bilibili_value,
                ratio_thread,
                rss_update_lock,
            ),
        )
        youtube_bilibili_rss_update_threads.append(thread)
        # 开始多线程
        thread.start()
    # 等待所有线程完成
    for thread in youtube_bilibili_rss_update_threads:
        thread.join()

    # 寻找错误原因
    def youtube_error(youtube_content, pattern_youtube_error):
        for (
            pattern_youtube_error_key,
            pattern_youtube_error_value,
        ) in pattern_youtube_error.items():
            if pattern_youtube_error_key in youtube_content:
                return pattern_youtube_error_value

    # 更新Youtube频道
    for youtube_key, youtube_value in gVar.channelid_youtube_ids.copy().items():
        youtube_response = gVar.channelid_youtube_rss[youtube_key]["content"]
        youtube_response_type = gVar.channelid_youtube_rss[youtube_key]["type"]
        # xml分类及存储
        if youtube_response is not None:
            if youtube_response_type == "dict":
                # 构建频道文件夹
                folder_build(youtube_key, "channel_audiovisual")
            else:
                if youtube_response_type == "html":
                    youtube_content = youtube_response.text
                elif youtube_response_type == "text":
                    youtube_content = youtube_response
                    write_log(f"YouTube频道 {youtube_value} 无法更新")
                else:
                    youtube_content = ""
                # 判断频道id是否正确
                if re.search(pattern_youtube404, youtube_content, re.DOTALL):
                    del gVar.channelid_youtube_ids[youtube_key]  # 删除错误ID
                    write_log(f"YouTube频道 {youtube_value} ID不正确无法获取")
                elif youtube_error_message := youtube_error(
                    youtube_content, pattern_youtube_error
                ):
                    del gVar.channelid_youtube_ids[youtube_key]  # 删除错误ID
                    write_log(f"YouTube频道 {youtube_value} {youtube_error_message}")
                else:
                    # 构建文件
                    file_save(youtube_content, f"{youtube_key}.txt", "channel_id")
                    # 构建频道文件夹
                    folder_build(youtube_key, "channel_audiovisual")
        else:
            if youtube_response_type == "text":
                del gVar.channelid_youtube_ids[youtube_key]
            write_log(f"YouTube频道 {youtube_value} 无法更新")
        # 更新进度条
        progress_bar(ratio_part, 0.1)

    # 更新哔哩哔哩频道
    for bilibili_key, bilibili_value in gVar.channelid_bilibili_ids.copy().items():
        bilibili_space = gVar.channelid_bilibili_rss[bilibili_key]["content"]
        bilibili_space_type = gVar.channelid_bilibili_rss[bilibili_key]["type"]
        # xml分类及存储
        if bilibili_space_type == "int":
            del gVar.channelid_bilibili_ids[bilibili_key]  # 删除错误ID
            write_log(f"BiliBili频道 {bilibili_value} ID不正确无法获取")
        elif bilibili_space_type == "json":
            write_log(f"BiliBili频道 {youtube_value} 无法更新")
            if bilibili_space == {}:
                del gVar.channelid_bilibili_ids[bilibili_key]
        else:
            # 构建文件
            file_save(bilibili_space, f"{bilibili_key}.json", "channel_id")
            # 构建频道文件夹
            folder_build(bilibili_key, "channel_audiovisual")
        # 更新进度条
        progress_bar(ratio_part, 0.1)



# podflow/message/original_rss_fail_print.py
# coding: utf-8

from podflow import gVar
from podflow.basic.write_log import write_log


# 打印无法保留原节目信息模块
def original_rss_fail_print(xmls_original_fail):
    channelid_ids = gVar.channelid_youtube_ids | gVar.channelid_bilibili_ids
    for item in xmls_original_fail:
        if item in channelid_ids.keys():
            write_log(
                f"RSS文件中不存在 {channelid_ids[item]} 无法保留原节目"
            )



# podflow/basic/file_save.py
# coding: utf-8

import os
import json


# 文件保存模块
def file_save(content, file_name, folder=None):
    # 如果指定了文件夹则将文件保存到指定的文件夹中
    if folder:
        file_path = os.path.join(os.getcwd(), folder, file_name)
    else:
        # 如果没有指定文件夹则将文件保存在当前工作目录中
        file_path = os.path.join(os.getcwd(), file_name)
    # 保存文件
    with open(file_path, "w", encoding="utf-8") as file:
        # 如果文件名中包含"."且文件类型为json，则将内容以json格式保存
        if "." in file_name and file_name.split(".")[-1] == "json":
            json.dump(content, file, ensure_ascii=False, indent=4)
        else:
            # 否则将内容以文本格式保存
            file.write(content)



# podflow/message/create_main_rss.py
# coding: utf-8

import time
from podflow import gVar
from podflow.youtube.build import youtube_xml_items
from podflow.bilibili.build import bilibili_xml_items
from podflow.message.get_media_name import get_media_name


def update_output_dir():
    output_dirs = []
    for format_value in gVar.video_id_update_format.values():
        if (
            isinstance(format_value, dict)
            and format_value["main"] not in gVar.video_id_failed
            and not format_value.get("finish",False)
        ):
            output_dirs.append(format_value["id"])
    return output_dirs


# 生成主rss模块
def create_main_rss():
    channelid_youtube_ids = gVar.channelid_youtube_ids
    channelid_bilibili_ids = gVar.channelid_bilibili_ids
    gVar.all_items = {
        key: {} for key in channelid_youtube_ids | channelid_bilibili_ids
    }
    all_channelid = list(gVar.all_items.keys())

    ratio_part = 0.6 / sum(gVar.xmls_quantity.values()) if all_channelid else 0

    while all_channelid:
        for index, output_dir in enumerate(all_channelid):
            if output_dir in update_output_dir():
                time.sleep(1)
            else:
                if output_dir in channelid_youtube_ids:
                    output_dir_youtube = channelid_youtube_ids[output_dir]
                    channelid_youtube_value = gVar.channelid_youtube[output_dir_youtube]
                    items = youtube_xml_items(output_dir, [ratio_part, 0.8])
                    items["DisplayRSSaddress"] = channelid_youtube_value[
                        "DisplayRSSaddress"
                    ]
                    items["QRcode"] = channelid_youtube_value["QRcode"]
                    items["ID_Name"] = output_dir_youtube
                    items["InmainRSS"] = channelid_youtube_value["InmainRSS"]
                    items["type"] = "youtube"
                    gVar.all_youtube_content_ytid[output_dir] = get_media_name(
                        "youtube", items["items"]
                    )
                    gVar.all_items[output_dir] = items
                elif output_dir in channelid_bilibili_ids:
                    output_dir_bilibili = channelid_bilibili_ids[output_dir]
                    channelid_bilibili_value = gVar.channelid_bilibili[
                        output_dir_bilibili
                    ]
                    items = bilibili_xml_items(output_dir, [ratio_part, 0.8])
                    items["DisplayRSSaddress"] = channelid_bilibili_value[
                        "DisplayRSSaddress"
                    ]
                    items["QRcode"] = channelid_bilibili_value["QRcode"]
                    items["ID_Name"] = output_dir_bilibili
                    items["InmainRSS"] = channelid_bilibili_value["InmainRSS"]
                    items["type"] = "bilibili"
                    gVar.all_bilibili_content_bvid[output_dir] = get_media_name(
                        "bilibili", items["items"]
                    )
                    gVar.all_items[output_dir] = items
                del all_channelid[index]
                break



# podflow/httpfs/app_bottle.py
# coding: utf-8

import os
import hashlib
import mimetypes
import pkg_resources
from datetime import datetime
import cherrypy
from bottle import Bottle, abort, redirect, request, static_file, response
from podflow import gVar
from podflow.upload.login import create
from podflow.basic.file_save import file_save
from podflow.basic.write_log import write_log
from podflow.httpfs.to_html import ansi_to_html
from podflow.upload.build_hash import build_hash
from podflow.upload.time_key import check_time_key
from podflow.httpfs.get_channelid import get_channelid


class bottle_app:
    # Bottle和Cherrypy初始化模块
    def __init__(self):
        self.app_bottle = Bottle()  # 创建 Bottle 应用
        self.bottle_print = []  # 存储打印日志
        self.setup_routes()  # 设置路由
        self.logname = "httpfs.log"  # 默认日志文件名
        self.http_fs = False

    def setup_routes(self, upload=False):
        # 设置/favicon.ico路由，回调函数为favicon
        self.app_bottle.route("/favicon.ico", callback=self.favicon)
        # 设置根路由，回调函数为home
        self.app_bottle.route("/", callback=self.home)
        # 设置/shutdown路由，回调函数为shutdown
        self.app_bottle.route("/shutdown", callback=self.shutdown)
        if upload:
            self.app_bottle.route("/newuser", callback=self.new_user)
            self.app_bottle.route("/login", callback=self.login)
            self.app_bottle.route("/upload", method="POST", callback=self.upload)
        else:
            self.app_bottle.route("/index", callback=self.index)
            self.app_bottle.route("/getid", method="POST", callback=self.getid)
            self.app_bottle.route("/<filename:path>", callback=self.serve_static)
            self.app_bottle.route("/message", callback=self.message)

    # 设置日志文件名及写入判断
    def set_logname(self, logname="httpfs.log", http_fs=False):
        self.logname = logname
        self.http_fs = http_fs

    # 判断token是否正确的验证模块
    def token_judgment(self, token, VALID_TOKEN="", filename="", foldername=""):
        # 判断 token 是否有效
        if foldername != "channel_audiovisual/":
            # 对于其他文件夹, 采用常规的 Token 验证
            return VALID_TOKEN == "" or token == VALID_TOKEN
        if (
            VALID_TOKEN == ""
            and token == hashlib.sha256(f"{filename}".encode()).hexdigest()
        ):  # 如果没有配置 Token, 则使用文件名的哈希值
            return True
        elif (
            token == hashlib.sha256(f"{VALID_TOKEN}/{filename}".encode()).hexdigest()
        ):  # 使用验证 Token 和文件名的哈希值
            return True
        else:
            return False

    # 添加至bottle_print模块
    def add_bottle_print(self, client_ip, filename, status):
        # 后缀
        suffixs = [".mp4", ".m4a", ".xml", ".ico"]
        # 设置状态码对应的颜色
        status_colors = {
            200: "\033[32m",  # 绿色 (成功)
            401: "\033[31m",  # 红色 (未经授权)
            404: "\033[35m",  # 紫色 (未找到)
            303: "\033[33m",  # 黄色 (重定向)
            206: "\033[36m",  # 青色 (部分内容)
        }
        # 默认颜色
        color = status_colors.get(status, "\033[0m")
        status = f"{color}{status}\033[0m"
        now_time = datetime.now().strftime("%H:%M:%S")
        client_ip = f"\033[34m{client_ip}\033[0m"
        if self.http_fs:
            write_log(
                f"{client_ip} {filename} {status}",
                None,
                False,
                True,
                None,
                self.logname,
            )
        for suffix in suffixs:
            filename = filename.replace(suffix, "")
        bottle_text = f"{now_time}|{client_ip} {filename} {status}"
        self.bottle_print.append(bottle_text)
        gVar.index_message["http"].append(ansi_to_html(bottle_text))

    # CherryPy 服务器打印模块
    def cherry_print(self, flag_judgment=True):
        # 如果flag_judgment为True，则将gVar.server_process_print_flag[0]设置为"keep"
        if flag_judgment:
            gVar.server_process_print_flag[0] = "keep"
        # 如果gVar.server_process_print_flag[0]为"keep"且self.bottle_print不为空，则打印日志
        if (
            gVar.server_process_print_flag[0] == "keep" and self.bottle_print
        ):  # 如果设置为保持输出, 则打印日志
            # 遍历self.bottle_print中的每个元素，并打印
            for info_print in self.bottle_print:
                print(info_print)
            # 清空self.bottle_print
            self.bottle_print.clear()

    # 输出请求日志的函数
    def print_out(self, filename, status):
        client_ip = request.remote_addr
        if client_port := request.environ.get("REMOTE_PORT"):
            client_ip = f"{client_ip}:{client_port}"
        if filename not in [
            "favicon.ico",
            "/",
            "shutdown",
            "newuser",
            "login",
        ]:
            bottle_channelid = (
                gVar.channelid_youtube_ids_original
                | gVar.channelid_bilibili_ids_original
                | {"channel_audiovisual/": "", "channel_rss/": ""}
            )  # 合并多个频道 ID
            for (
                bottle_channelid_key,
                bottle_channelid_value,
            ) in bottle_channelid.items():
                filename = filename.replace(
                    bottle_channelid_key, bottle_channelid_value
                )  # 替换频道路径
                if status == 200 and request.headers.get(
                    "Range"
                ):  # 如果是部分请求, 则返回 206 状态
                    status = 206
        self.add_bottle_print(client_ip, filename, status)  # 输出日志
        self.cherry_print(False)

    # 主路由处理根路径请求
    def home(self):
        VALID_TOKEN = gVar.config["token"]  # 从配置中读取主验证 Token
        token = request.query.get("token")  # 获取请求中的 Token

        if self.token_judgment(token, VALID_TOKEN):  # 验证 Token
            self.print_out("/", 303)  # 如果验证成功, 输出 200 状态
            return redirect("https://github.com/gruel-zxz/podflow")  # 返回正常响应
        else:
            self.print_out("/", 401)  # 如果验证失败, 输出 401 状态
            abort(401, "Unauthorized: Invalid Token")  # 返回未经授权错误

    # 路由处理关闭服务器的请求
    def shutdown(self):
        Shutdown_VALID_TOKEN = "shutdown"
        Shutdown_VALID_TOKEN += datetime.now().strftime("%Y%m%d%H%M%S")
        Shutdown_VALID_TOKEN += os.urandom(32).hex()
        Shutdown_VALID_TOKEN = hashlib.sha256(
            Shutdown_VALID_TOKEN.encode()
        ).hexdigest()  # 用于服务器关闭的验证 Token
        token = request.query.get("token")  # 获取请求中的 Token

        if self.token_judgment(
            token, Shutdown_VALID_TOKEN
        ):  # 验证 Token 是否为关闭用的 Token
            self.print_out("shutdown", 200)  # 如果验证成功, 输出 200 状态
            cherrypy.engine.exit()  # 使用 CherryPy 提供的停止功能来关闭服务器
            return "Shutting down..."  # 返回关机响应
        else:
            self.print_out("shutdown", 401)  # 如果验证失败, 输出 401 状态
            abort(401, "Unauthorized: Invalid Token")  # 返回未经授权错误

    # 路由处理 favicon 请求
    def favicon(self):
        self.print_out("favicon.ico", 303)  # 输出访问 favicon 的日志
        return redirect(
            "https://raw.githubusercontent.com/gruel-zxz/podflow/main/Podflow.png"
        )  # 重定向到图标 URL

    # 路由处理静态文件请求
    def serve_static(self, filename):
        VALID_TOKEN = gVar.config["token"]  # 从配置中读取主验证 Token
        # 定义要共享的文件路径
        bottle_filename = gVar.config["filename"]  # 从配置中读取文件名
        shared_files = {
            bottle_filename.lower(): f"{bottle_filename}.xml",  # 文件路径映射, 支持大小写不敏感的文件名
            f"{bottle_filename.lower()}.xml": f"{bottle_filename}.xml",  # 同上, 支持带 .xml 后缀
        }
        token = request.query.get("token")  # 获取请求中的 Token

        # 文件是否存在检查的函数
        def file_exist(token, VALID_TOKEN, filename, foldername=""):
            # 验证 Token
            if self.token_judgment(
                token, VALID_TOKEN, filename, foldername
            ):  # 验证 Token
                # 如果文件存在, 返回文件
                if os.path.exists(filename):  # 如果文件存在, 返回文件
                    self.print_out(filename, 200)
                    # 设置正确的 Content-Type 头部
                    content_type, _ = mimetypes.guess_type(filename)
                    # 如果无法自动猜测出正确的 Content-Type，手动指定
                    if not content_type:
                        if filename.endswith(".xml"):
                            content_type = "application/xml"
                        elif filename.endswith(".m4a"):
                            content_type = "audio/mp4"
                        elif filename.endswith(".mp4"):
                            content_type = "video/mp4"
                        else:
                            content_type = "application/octet-stream"  # 默认文件类型

                    # 返回静态文件并附加正确的 Content-Type
                    return static_file(filename, root=".", mimetype=content_type)
                else:  # 如果文件不存在, 返回 404 错误
                    self.print_out(filename, 404)
                    abort(404, "File not found")
            else:  # 如果 Token 验证失败, 返回 401 错误
                self.print_out(filename, 401)
                abort(401, "Unauthorized: Invalid Token")

        # 处理不同的文件路径
        if filename in ["channel_audiovisual/", "channel_rss/"]:
            self.print_out(filename, 404)
            abort(404, "File not found")
        elif filename.startswith("channel_audiovisual/"):
            return file_exist(token, VALID_TOKEN, filename, "channel_audiovisual/")
        elif filename.startswith("channel_rss/") and filename.endswith(".xml"):
            return file_exist(token, VALID_TOKEN, filename)
        elif filename.startswith("channel_rss/"):
            return file_exist(token, VALID_TOKEN, f"{filename}.xml")
        elif filename.lower() in shared_files:
            return file_exist(token, VALID_TOKEN, shared_files[filename.lower()])
        else:
            self.print_out(filename, 404)  # 如果文件路径未匹配, 返回 404 错误
            abort(404, "File not found")

    # 路由获取账号密码请求
    def new_user(self):
        # 生成一个用于上传非一次性项目的账户密码，该密码需要保存
        seed = "We need to generate an account password for uploading non one-time items that need to be saved."
        token = request.query.get("token")  # 获取请求中的 Token
        response.content_type = "application/json"

        if check_time_key(token, seed):  # 验证 Token
            username, password = create()  # 生成用户名和密码
            self.print_out("newuser", 200)
            return {
                "code": 0,
                "message": "Get New Username And Password Success",
                "data": {
                    "username": username,
                    "password": password,
                },
            }
        else:
            self.print_out("newuser", 401)
            return {
                "code": -1,
                "message": "Unauthorized: Invalid Token",
            }

    # 路由处理登陆请求
    def login(self):
        # 获取上传的数据
        upload_data = gVar.upload_data
        # 获取用户名
        username = request.query.get("username")
        # 获取密码
        password = request.query.get("password")
        # 判断用户名是否在上传的数据中
        if username in upload_data:
            # 判断密码是否正确
            if upload_data[username] == password:
                # 打印登录成功
                self.print_out("login", 200)
                # 返回登录成功的信息
                return {
                    "code": 0,
                    "message": "Login Success",
                }
            else:
                # 打印密码错误
                self.print_out("login", 401)
                # 返回密码错误的信息
                return {
                    "code": -3,
                    "message": "Password Error",
                }
        else:
            # 打印用户名错误
            self.print_out("login", 401)
            # 返回用户名错误的信息
            return {
                "code": -2,
                "message": "Username Error",
            }

    # 文件上传处理请求
    def upload(self):
        # 获取上传数据配置(存储用户名和密码)
        upload_data = gVar.upload_data
        # 从请求参数中获取用户名，默认为空字符串
        username = request.query.get("username", "")
        # 从请求参数中获取密码，默认为空字符串
        password = request.query.get("password", "")
        upload_hash = request.query.get("hash", "")
        channelid = request.query.get("channel_id", "")

        # 验证用户是否存在
        if username not in upload_data:
            self.print_out("login", 401)
            return {
                "code": -2,
                "message": "Username Error",
            }
        # 验证密码是否正确
        if upload_data[username] != password:
            self.print_out("login", 401)
            return {
                "code": -3,
                "message": "Password Error",
            }
        # 从请求中获取上传的文件对象
        upload_file = request.files.get("file")
        # 检查是否有文件被上传
        if not upload_file:
            # 打印错误信息并返回错误码
            self.print_out("upload", 404)
            return {
                "code": -4,
                "message": "No File Provided",
            }
        # 判断文件是否完整
        if upload_hash != build_hash(upload_file):
            self.print_out("upload", 401)
            return {
                "code": -5,
                "message": "Incomplete File",
            }
        if not channelid:
            # 打印错误信息并返回错误码
            self.print_out("upload", 404)
            return {
                "code": -6,
                "message": "ChannelId Does Not Exist",
            }
        # 获取上传文件的原始文件名
        filename = upload_file.filename
        name = filename.split(".")[0]
        suffix = filename.split(".")[1]
        if suffix in ["mp4", "m4a"]:
            self.print_out("upload", 404)
            return {
                "code": -6,
                "message": "File Format Error",
            }
        address = f"/channel_audiovisual/{channelid}"
        if os.path.exists(address):
            file_list = os.listdir(address)
        else:
            file_list = []
        num = 0
        while True:
            if num != 0:
                filename = f"{name}.{num}.{suffix}"
            if filename in file_list:
                with open(f"{address}/{filename}", "rb") as original_file:
                    if upload_hash == build_hash(original_file):
                        self.print_out("upload", 200)
                        return {
                            "code": 1,
                            "message": "The Same File Exists",
                            "data": {
                                "filename": filename,
                            },
                        }
                    num += 1
            else:
                file_save(upload_file, filename, address)
                # 打印成功信息并返回成功码
                self.print_out("upload", 200)
                return {
                    "code": 0,
                    "message": "Upload Success",
                    "data": {
                        "filename": filename,
                    },
                }

    # 使用pkg_resources获取模板文件路径
    def index(self):
        template_path = pkg_resources.resource_filename('podflow', 'templates/index.html')
        with open(template_path, 'r', encoding="UTF-8") as f:
            html_content = f.read()
        self.print_out("index", 200)
        return html_content

    # 获取 JSON 数据，Bottle 会自动解析请求体中的 JSON 数据
    def getid(self):
        if getid_data := request.json:
            content = getid_data.get("content", "")
        else:
            content = ""
        response_message = get_channelid(content)
        self.print_out("channelid", 200)
        # 设置响应头为 application/json
        response.content_type = 'application/json'
        return {"response": response_message}

    # 处理消息的接收和发送。
    def message(self):
        response.content_type = 'application/json'
        return gVar.index_message # 获取消息列表


bottle_app_instance = bottle_app()



# setup.py
# coding: utf-8

from setuptools import setup, find_packages

setup(
    name="podflow",
    version="20250417.1",
    author="gruel_zxz",
    author_email="zhuxizhouzxz@gmail.com",
    description="A podcast server that includes YouTube and BiliBili",
    long_description=open('README.md', 'r', encoding='utf-8').read(),
    long_description_content_type="text/markdown",
    url="https://github.com/gruel-zxz/podflow",
    packages=find_packages(),
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'Podflow=podflow.main:main',
            'podflow=podflow.main:main',
            'PodFlow=podflow.main:main',
            'PODFLOW=podflow.main:main',
        ],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.8",
    install_requires=[
        "astral>=3.2", 
        "bottle>=0.13.2",
        "yt-dlp>=2025.3.31",
        "chardet>=5.2.0",
        "cherrypy>=18.10.0",
        "pyqrcode>=1.2.1",
        "requests>=2.32.3",
        "pycryptodome>=3.21.0",
        "ffmpeg-python>=0.2.0",
        "BeautifulSoup4>=4.13.3",
    ],
)



# podflow/netscape/bulid_netscape.py
# coding: utf-8

from podflow.basic.file_save import file_save


# 生成Netscape_HTTP_Cookie模块
def bulid_netscape(file_name, cookie=None):
    if cookie is None:
        cookie = {}
    if "bilibili" in file_name:
        cookie_jar = f'''# Netscape HTTP Cookie File
# This file is generated by yt-dlp.  Do not edit.

.bilibili.com	TRUE	/	FALSE	0	SESSDATA	{cookie.get("SESSDATA", "")}
.bilibili.com	TRUE	/	FALSE	0	bili_jct	{cookie.get("bili_jct", "")}
.bilibili.com	TRUE	/	FALSE	0	DedeUserID	{cookie.get("DedeUserID", "")}
.bilibili.com	TRUE	/	FALSE	0	DedeUserID__ckMd5	{cookie.get("DedeUserID__ckMd5", "")}
.bilibili.com	TRUE	/	FALSE	0	sid	{cookie.get("sid", "")}
.bilibili.com	TRUE	/	FALSE	0	buvid3	{cookie.get("buvid3", "")}
.bilibili.com	TRUE	/	FALSE	0	b_nut	{cookie.get("b_nut", "")}'''
    elif "youtube" in file_name:
        cookie_jar = f'''# Netscape HTTP Cookie File
# This file is generated by yt-dlp.  Do not edit.

.youtube.com	TRUE	/	TRUE	0	__Secure-3PSID	{cookie.get("__Secure-3PSID", "")}
.youtube.com	TRUE	/	TRUE	0	__Secure-1PSIDTS	{cookie.get("__Secure-1PSIDTS", "")}
.youtube.com	TRUE	/	TRUE	0	SAPISID	{cookie.get("SAPISID", "")}
.youtube.com	TRUE	/	TRUE	0	__Secure-1PSIDCC	{cookie.get("__Secure-1PSIDCC", "")}
.youtube.com	TRUE	/	TRUE	0	SSID	{cookie.get("SSID", "")}
.youtube.com	TRUE	/	TRUE	0	__Secure-1PAPISID	{cookie.get("__Secure-1PAPISID", "")}
.youtube.com	TRUE	/	TRUE	0	__Secure-1PSID	{cookie.get("__Secure-1PSID", "")}
.youtube.com	TRUE	/	TRUE	0	__Secure-3PAPISID	{cookie.get("__Secure-3PAPISID", "")}
.youtube.com	TRUE	/	TRUE	0	__Secure-3PSIDCC	{cookie.get("__Secure-3PSIDCC", "")}
.youtube.com	TRUE	/	TRUE	0	__Secure-3PSIDTS	{cookie.get("__Secure-3PSIDTS", "")}
.youtube.com	TRUE	/	TRUE	0	LOGIN_INFO	{cookie.get("LOGIN_INFO", "")}
.youtube.com	TRUE	/	FALSE	0	PREF	{cookie.get("PREF", "")}
.youtube.com	TRUE	/	TRUE	0	YSC	{cookie.get("YSC", "")}
.youtube.com	TRUE	/	TRUE	0	VISITOR_INFO1_LIVE	{cookie.get("VISITOR_INFO1_LIVE", "")}
.youtube.com	TRUE	/	TRUE	0	VISITOR_PRIVACY_METADATA	{cookie.get("VISITOR_PRIVACY_METADATA", "")}'''
    else:
        cookie_jar = '''# Netscape HTTP Cookie File
# This file is generated by yt-dlp.  Do not edit.'''
    file_save(cookie_jar, f"{file_name}.txt", "channel_data")



# podflow/config/get_config.py
# coding: utf-8

import os
import sys
import json
from podflow import default_config
from podflow.basic.write_log import write_log
from podflow.basic.time_print import time_print


# 获取配置信息config模块
def get_config(file_name="config.json"):
    # 检查当前文件夹中是否存在config文件
    if not os.path.exists(file_name):
        if os.path.exists("config.json"):
            write_log(f"不存在配置文件{file_name}, 将使用原始配置文件")
            file_name = "config.json"
        else:
            # 如果文件不存在, 创建并写入默认字典
            with open("config.json", "w") as file:
                json.dump(default_config, file, indent=4)
            write_log("不存在配置文件, 已新建, 默认频道")
            return default_config
    # 如果文件存在, 读取字典并保存到config变量中
    try:
        with open(file_name, "r", encoding="utf-8") as file:
            config = json.load(file)
        time_print("已读取配置文件")
        return config
    # 如果config格式有问题, 停止运行并报错
    except Exception as config_error:
        write_log(f"配置文件有误, 请检查{file_name}, {str(config_error)}")
        sys.exit(0)



# podflow/main_podcast.py
# coding: utf-8

import sys
import json
import time
import urllib
import subprocess

import cherrypy

# 基本功能模块
from podflow import gVar, parse
from podflow.basic.split_dict import split_dict
from podflow.basic.time_print import time_print

# 网络和 HTTP 模块
from podflow.httpfs.browser import open_url
from podflow.httpfs.port_judge import port_judge
from podflow.httpfs.progress_bar import progress_update
from podflow.httpfs.app_bottle import bottle_app_instance

# 下载和视频处理模块
from podflow.ffmpeg_judge import ffmpeg_judge
from podflow.download.delete_part import delete_part
from podflow.download_and_build import download_and_build

# RSS 和消息处理模块
from podflow.message.save_rss import save_rss
from podflow.message.get_original_rss import get_original_rss
from podflow.message.get_video_format import get_video_format
from podflow.message.optimize_download import optimize_download
from podflow.message.original_rss_fail_print import original_rss_fail_print
from podflow.message.update_information_display import update_information_display
from podflow.message.update_youtube_bilibili_rss import update_youtube_bilibili_rss

# 登录模块
from podflow.bilibili.login import get_bilibili_data
from podflow.youtube.login import get_youtube_cookie

# 配置和图标模块
from podflow.config.channge_icon import channge_icon
from podflow.config.build_original import build_original

# 制作和修改文件模块
from podflow.makeup.make_up_file import make_up_file
from podflow.makeup.make_up_file_mod import make_up_file_mod
from podflow.makeup.del_makeup_format_fail import del_makeup_format_fail
from podflow.makeup.make_up_file_format_mod import make_up_file_format_mod

# 移除模块
from podflow.remove.remove_dir import remove_dir
from podflow.remove.remove_file import remove_file

# 处理 YouTube 信息模块
from podflow.youtube.build import print_fail_youtube

# 长期媒体进行上传模块
from podflow.upload.login import login_upload
from podflow.upload.add_upload import add_upload
from podflow.upload.update_upload import update_upload
from podflow.upload.linked_client import connect_upload_server
from podflow.upload.get_upload_original import get_upload_original


def main_podcast():
    # 判断是否安装ffmpeg
    ffmpeg_judge()
    # 初始化
    build_original()
    # http共享
    port = gVar.config["port"]
    hostip = "0.0.0.0"
    if port_judge(hostip, port):
        # 设置路由
        bottle_app_instance.setup_routes(upload=False)
        # 设置logname
        bottle_app_instance.set_logname(
            logname="httpfs.log",
            http_fs=gVar.config["httpfs"],
        )
        # 启动 CherryPy 服务器
        cherrypy.tree.graft(
            bottle_app_instance.app_bottle
        )  # 将 Bottle 应用嵌入到 CherryPy 中
        cherrypy.config.update(
            {
                "global": {
                    "tools.sessions.on": True,  # 启用会话支持
                    "server.socket_host": hostip,  # 监听所有 IP 地址
                    "server.socket_port": port,  # 设置监听端口
                    "log.screen": False,  # 禁用屏幕日志输出
                    "log.access_file": "",  # 关闭访问日志
                    "log.error_file": "",  # 关闭错误日志
                }
            }
        )
        cherrypy.engine.start()  # 启动 CherryPy 服务器
        time_print(f"HTTP服务器启动, 端口: \033[32m{port}\033[0m")
        if parse.index:
            open_url(f"{gVar.config['address']}/index")
        if parse.httpfs:  # HttpFS参数判断, 是否继续运行
            cherrypy.engine.block()  # 阻止程序退出, 保持HTTP服务运行
            sys.exit(0)
    else:
        time_print(f"HTTP服务器端口: \033[32m{port}\033[0m, \033[31m被占用\033[0m")
        if parse.httpfs:
            sys.exit(0)
    # 主流程
    while parse.update_num > 0 or parse.update_num == -1:  # 循环主更新
        # 主进度条
        progress_update(0, refresh=1)
        # 暂停进程打印
        gVar.server_process_print_flag[0] = "pause"
        # 获取YouTube cookie
        gVar.youtube_cookie = get_youtube_cookie(gVar.channelid_youtube_ids_original)
        progress_update(0.01 ,num=0.0049)
        # 更新哔哩哔哩data
        gVar.channelid_bilibili_ids, gVar.bilibili_data = get_bilibili_data(
            gVar.channelid_bilibili_ids_original
        )
        progress_update(0.02, num=0.0049)
        # 恢复进程打印
        bottle_app_instance.cherry_print()
        # 获取原始xml字典和rss文本
        gVar.xmls_original, gVar.hash_rss_original, gVar.xmls_original_fail  = get_original_rss()
        progress_update(0.025, num=-0.0024)
        # 暂停进程打印
        gVar.server_process_print_flag[0] = "pause"
        # 连接上传服务器
        upload_url = connect_upload_server()
        progress_update(0.03, num=0.0024)
        # 恢复进程打印
        bottle_app_instance.cherry_print()
        # 登陆上传服务器
        if upload_url:
            upload_json = login_upload(upload_url)
            if upload_json:
                gVar.config["upload"] = False
        else:
            gVar.config["upload"] = False
        progress_update(0.035, num=0.0024)
        # 初始化原始上传信息
        get_upload_original()
        progress_update(0.04)
        # 更新Youtube和哔哩哔哩频道xml
        update_youtube_bilibili_rss()
        progress_update(0.1)
        # 判断是否有更新内容
        if gVar.channelid_youtube_ids_update or gVar.channelid_bilibili_ids_update:
            gVar.update_generate_rss = True
        if gVar.update_generate_rss:
            # 根据日出日落修改封面(只适用原封面)
            channge_icon()
            progress_update(0.11, num=0.0049)
            # 输出需要更新的信息
            update_information_display(
                gVar.channelid_youtube_ids_update,
                gVar.youtube_content_ytid_update,
                gVar.youtube_content_ytid_backward_update,
                "YouTube",
            )
            update_information_display(
                gVar.channelid_bilibili_ids_update,
                gVar.bilibili_content_bvid_update,
                gVar.bilibili_content_bvid_backward_update,
                "BiliBili",
            )
            progress_update(0.12)
            # 暂停进程打印
            gVar.server_process_print_flag[0] = "pause"
            # 获取视频格式信息
            get_video_format()
            progress_update(0.199)
            # 恢复进程打印
            bottle_app_instance.cherry_print()
            # 优化下载顺序
            optimize_download()
            # 删除中断下载的媒体文件
            if gVar.config["delete_incompletement"]:
                delete_part(gVar.channelid_youtube_ids | gVar.channelid_bilibili_ids)
            progress_update(0.20, refresh=2)
            # 暂停进程打印
            gVar.server_process_print_flag[0] = "pause"
            # 下载并构建YouTube和哔哩哔哩视频
            download_and_build()
            progress_update(0.8)
            # 添加新媒体至上传列表
            add_upload()
            progress_update(0.81, num=0.0049)
            # 恢复进程打印
            bottle_app_instance.cherry_print()
            # 打印无法保留原节目信息
            original_rss_fail_print(gVar.xmls_original_fail)
            progress_update(0.82, num=0.0049)
            # 打印无法获取youtube信息
            print_fail_youtube()
            progress_update(0.83, num=0.0049)
            if gVar.config["remove_media"]:
                # 删除不在rss中的媒体文件
                remove_file()
                # 删除已抛弃的媒体文件夹
                remove_dir()
            progress_update(0.84)
            # 补全缺失媒体文件到字典
            make_up_file()
            progress_update(0.85)
            # 按参数获取需要补全的最大个数
            gVar.make_up_file_format = split_dict(
                gVar.make_up_file_format,
                gVar.config["completion_count"],
                True,
            )[0]
            progress_update(0.86, num=0.0049)
            # 暂停进程打印
            gVar.server_process_print_flag[0] = "pause"
            # 补全在rss中缺失的媒体格式信息
            make_up_file_format_mod()
            progress_update(0.90, num=0.0049)
            # 恢复进程打印
            bottle_app_instance.cherry_print()
            progress_update(0.91, num=0.0049)
            # 删除无法补全的媒体
            del_makeup_format_fail()
            progress_update(0.92, num=0.0049)
            # 暂停进程打印
            gVar.server_process_print_flag[0] = "pause"
            # 保存rss文件模块
            save_rss()
            progress_update(0.93, num=0.0049)
            # 下载补全Youtube和哔哩哔哩视频模块
            make_up_file_mod()
            progress_update(0.99, num=0.0099)
            # 恢复进程打印
            bottle_app_instance.cherry_print()
            # 更新并保存上传列表
            update_upload()
            progress_update(1, refresh=3)
        else:
            time_print("频道无更新内容")
        # 清空变量内数据
        gVar.channelid_youtube_ids_update.clear()  # 需更新的YouTube频道字典
        gVar.youtube_content_ytid_update.clear()  # 需下载YouTube视频字典
        gVar.youtube_content_ytid_backward_update.clear()  # 向后更新需下载YouTube视频字典
        gVar.channelid_youtube_rss.clear()  # YouTube频道最新Rss Response字典
        gVar.channelid_bilibili_ids_update.clear()  # 需更新的哔哩哔哩频道字典
        gVar.bilibili_content_bvid_update.clear()  # 需下载哔哩哔哩视频字典
        gVar.channelid_bilibili_rss.clear()  # 哔哩哔哩频道最新Rss Response字典
        gVar.bilibili_content_bvid_backward_update.clear()  # 向后更新需下载哔哩哔哩视频字典
        gVar.video_id_failed.clear()  # YouTube&哔哩哔哩视频下载失败列表
        gVar.video_id_update_format.clear()  # YouTube&哔哩哔哩视频下载的详细信息字典
        gVar.hash_rss_original = ""  # 原始rss哈希值文本
        gVar.xmls_original.clear()  # 原始xml信息字典
        gVar.xmls_original_fail.clear()  # 未获取原始xml频道列表
        gVar.xmls_quantity.clear()  # xml数量字典
        gVar.youtube_xml_get_tree.clear()  # YouTube频道简介和图标字典
        gVar.all_youtube_content_ytid.clear()  # 所有YouTube视频id字典
        gVar.all_bilibili_content_bvid.clear()  # 所有哔哩哔哩视频id字典
        gVar.all_items.clear()  # 更新后所有item明细列表
        gVar.overall_rss = ""  # 更新后的rss文本
        gVar.make_up_file_format.clear()  # 补全缺失媒体字典
        gVar.make_up_file_format_fail.clear()  # 补全缺失媒体失败字典
        # 将需要更新转为否
        gVar.update_generate_rss = False
        if parse.update_num != -1:
            parse.update_num -= 1
        if parse.argument == "a-shell":
            shortcuts_input = urllib.parse.quote(json.dumps(gVar.shortcuts_url))
            openserver_process = subprocess.Popen(
                [
                    "open",
                    f"shortcuts://run-shortcut?name=Podflow&input=text&text={shortcuts_input}",
                ]
            )
            # 延时
            time.sleep(60 + len(gVar.shortcuts_url) * 5)
            openserver_process.terminate()
            break
        elif parse.update_num == 0:
            break
        else:
            # 延时
            time.sleep(parse.time_delay)
    # 关闭CherryPy服务器
    time_print("Podflow运行结束")
    cherrypy.engine.exit()



# podflow/httpfs/progress_bar.py
# coding: utf-8

import random
from podflow import gVar


def progress_update(
    ratio,
    added=False,
    refresh=0,
    num=0
):
    state = {
        1: "准备中",
        2: "构建中",
        3: "已完成",
    }
    if num != 0:
        ratio +=random.uniform(0, num)
    if added:
        ratio += gVar.index_message["schedule"][1]
    gVar.index_message["schedule"][1] = ratio
    if refresh != 0:
        gVar.index_message["schedule"][0] = state[refresh]


def progress_bar(ratio_part, maximum):
    ratio = gVar.index_message["schedule"][1] + ratio_part
    if ratio > maximum:
        ratio = maximum
    progress_update(ratio)



# podflow/basic/get_duration.py
# coding: utf-8

import math
import ffmpeg
from podflow.basic.write_log import write_log


# 获取已下载视频时长模块
def get_duration(file_path):
    try:
        # 调用ffmpeg获取视频文件的时长信息
        probe = ffmpeg.probe(file_path)
        duration = float(probe['format']['duration'])
        return math.ceil(duration)
    except ffmpeg.Error as e:
        error_note = e.stderr.decode('utf-8').splitlines()[-1]
        write_log(f"\033[31mError:\033[0m {error_note}")
